# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""simple helper function to link io module and XASObject"""

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "07/16/2019"


from est.io import read_xas, write_xas, get_xasproc
from est.core.types import XASObject
from silx.io.url import DataUrl
from est.core.types import Dim
from est.units import ur, convert_to
from typing import Union
from est.io.utils.information import InputInformation
import h5py
import logging

_logger = logging.getLogger(__name__)

DEFAULT_SPECTRA_PATH = "/data/NXdata/data"

DEFAULT_CHANNEL_PATH = "/data/NXdata/Channel"

DEFAULT_CONF_PATH = "/configuration"


def read(
    spectra_url,
    channel_url,
    dimensions=(Dim.DIM_2, Dim.DIM_1, Dim.DIM_0),
    config_url=None,
    energy_unit=ur.eV,
):
    """

    :param DataUrl spectra_url: data url to the spectra
    :param DataUrl channel_url: data url to the channel / energy
    :param DataUrl config_url: data url to the process configuration
    :param dimensions: way the data has been stored.
                       Usually is (X, Y, channels) of (Channels, Y, X).
                       If None, by default is considered to be (Z, Y, X)
    :type: tuple
    :return:
    :rtype: XASObject
    """
    dimensions_ = dimensions
    if dimensions_ is None:
        dimensions_ = (Dim.DIM_2, Dim.DIM_1, Dim.DIM_0)
    reader = XASReader()
    return reader.read_frm_url(
        InputInformation(
            spectra_url=spectra_url,
            channel_url=channel_url,
            config_url=config_url,
            dimensions=dimensions_,
            energy_unit=energy_unit,
        )
    )


def read_frm_file(
    file_path,
    energy_unit=ur.eV,
    dimensions: tuple = (Dim.DIM_2, Dim.DIM_1, Dim.DIM_0),
    columns_names: Union[None, dict] = None,
):
    """

    :param str file_path: path to the file containing the spectra. Must ba a
                          .dat file that pymca can handle or a .h5py with
                          default path
    :param tuple dimensions: dimensions of the input data. For ASCII file can
                             be (X, Y) or (Y, X)
    :param Union[None,tuple] columns: name of the column to take for .dat...
                                      files.
    :return XasObject created from the input
    :rtype: XASObject
    """
    if file_path in (None, ""):
        return
    reader = XASReader()
    return reader.read_from_file(
        file_path=file_path,
        energy_unit=energy_unit,
        dimensions=dimensions,
        columns_names=columns_names,
    )


class XASReader(object):
    """Simple reader of a xas file"""

    @staticmethod
    def read_frm_url(input_information):
        sp, en, conf = read_xas(information=input_information)
        return XASObject(spectra=sp, energy=en, configuration=conf)

    @staticmethod
    def read_from_file(
        file_path,
        energy_unit=ur.eV,
        dimensions=(Dim.DIM_2, Dim.DIM_1, Dim.DIM_0),
        columns_names=None,
    ):
        """

        :param str file_path:
        :return: `.XASObject`
        """
        # TODO: we should be able to avoid calling the creation of an InputInformation
        if file_path.endswith((".dat", ".csv")):
            return XASReader.read_frm_url(
                InputInformation(
                    spectra_url=DataUrl(file_path=file_path, scheme="PyMca"),
                    channel_url=DataUrl(file_path=file_path, scheme="PyMca"),
                    energy_unit=energy_unit,
                    dimensions=dimensions,
                    columns_names=columns_names,
                )
            )
        elif file_path.endswith(".xmu"):
            return XASReader.read_frm_url(
                InputInformation(
                    spectra_url=DataUrl(file_path=file_path, scheme="larch"),
                    channel_url=DataUrl(file_path=file_path, scheme="larch"),
                    energy_unit=energy_unit,
                    dimensions=dimensions,
                    columns_names=columns_names,
                )
            )
        elif h5py.is_hdf5(file_path):
            return XASReader.read_frm_url(
                InputInformation(
                    spectra_url=DataUrl(
                        file_path=file_path,
                        scheme="silx",
                        data_path=DEFAULT_SPECTRA_PATH,
                    ),
                    channel_url=DataUrl(
                        file_path=file_path,
                        scheme="silx",
                        data_path=DEFAULT_CHANNEL_PATH,
                    ),
                    config_url=DataUrl(
                        file_path=file_path, scheme="silx", data_path="configuration"
                    ),
                    energy_unit=energy_unit,
                    dimensions=dimensions,
                )
            )
        else:
            raise ValueError(
                "file type {} not managed, unable to load".format(file_path)
            )

    __call__ = read_from_file


class XASWriter(object):
    """
    class to write the output file. In this case we need a class in order to
    setup the output file before
    """

    def __init__(self):
        self._output_file = None

    @property
    def output_file(self):
        return self._output_file

    @output_file.setter
    def output_file(self, file_):
        self._output_file = file_

    def set_properties(self, properties):
        if "_output_file_setting" in properties:
            self._output_file = properties["_output_file_setting"]

    def dump_xas(self, xas_obj, write_process=True):
        """
        write the XASObject into a hdf5 file.

        :param xas_obj: object to be stored
        :type: Union[:class:`.XASObject`,dict]
        :param bool write_process: if True then store the process flow in the
            same file.
        """
        if isinstance(xas_obj, dict):
            _xas_obj = XASObject.from_dict(xas_obj)
        else:
            _xas_obj = xas_obj

        if not self._output_file:
            _logger.warning(
                "no output file defined, please give path to the" "output file"
            )
            self._output_file = input()

        _logger.info("dump xas obj to {}".format(self._output_file))

        # write raw data
        write_xas(
            h5_file=self._output_file,
            energy=_xas_obj.energy,
            mu=_xas_obj.absorbed_beam(),
            entry=_xas_obj.entry,
        )

        if write_process is True:
            if len(get_xasproc(self._output_file, entry=_xas_obj.entry)) > 0:
                _logger.warning(
                    "removing previous process registred. They are no " "more accurate"
                )
                _xas_obj.clean_process_flow()

            # write process flow
            _xas_obj.copy_process_flow_to(self._output_file)

    __call__ = dump_xas



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "13/08/2019"


from silx.gui import qt


class _ParameterWindowContainer(qt.QWidget):
    """Embed the larch parameters windows (should contains getParameters and
    setParameters and have a sigChanged signal) and add it a 'manual' update
    mode for convenience
    """

    sigChanged = qt.Signal()
    """Signal emitted when some computation is required"""

    def __init__(self, parent, parametersWindow):
        assert parametersWindow is not None
        super().__init__(parent=parent)
        self.setLayout(qt.QGridLayout())
        self._mainwidget = parametersWindow(parent=self)
        assert hasattr(self._mainwidget, "setParameters")
        assert hasattr(self._mainwidget, "getParameters")
        self.layout().addWidget(self._mainwidget, 0, 0, 1, 3)
        self._autoCB = qt.QCheckBox("manual update", parent=self)
        self.layout().addWidget(self._autoCB, 1, 1, 1, 1)
        self._autoCB.setToolTip(
            "if activated will wait until you press the "
            '"update" button to launch processing. '
            "Otherwise executed for each modification in "
            "the paramters"
        )
        self._manualUpdatePB = qt.QPushButton("update")
        self.layout().addWidget(self._manualUpdatePB, 1, 2, 1, 1)

        # setup
        self._manualUpdatePB.setVisible(False)

        # expose API
        self.setParameters = self._mainwidget.setParameters
        self.getParameters = self._mainwidget.getParameters

        # connect signal / slot
        self._autoCB.toggled.connect(self._manualUpdatePB.setVisible)
        self._manualUpdatePB.pressed.connect(self._update)

        if hasattr(self._mainwidget, "sigChanged"):
            # manage larch windows
            _sig = self._mainwidget.sigChanged
        elif hasattr(self._mainwidget, "sigFTParametersSignal"):
            _sig = self._mainwidget.sigFTParametersSignal
        elif hasattr(self._mainwidget, "sigNormalizationParametersSignal"):
            _sig = self._mainwidget.sigNormalizationParametersSignal
        elif hasattr(self._mainwidget, "sigXASNormalizationParametersSignal"):
            _sig = self._mainwidget.sigXASNormalizationParametersSignal
        elif hasattr(self._mainwidget, "sigPostEdgeParametersSignal"):
            _sig = self._mainwidget.sigPostEdgeParametersSignal
        else:
            raise ValueError("window not recognized")

        _sig.connect(self._filteredUpdate)

    def _filteredUpdate(self, *args, **kwargs):
        """call _update only if in automatic update mode"""
        if not self._autoCB.isChecked():
            self._update()

    def _update(self, *args, **kwargs):
        self.sigChanged.emit()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "03/07/2019"

from silx.gui.dialog.DataFileDialog import DataFileDialog
from silx.io.url import DataUrl
from silx.gui import qt
from est.core.io import read as read_xas, read_frm_file
from est.io import InputType
import logging
from est.core.types import Dim
from est.gui.unit.energy import EnergyUnitSelector
from est.core.utils.symbol import MU_CHAR
from typing import Iterable
from silx.io.spech5 import SpecFile

_logger = logging.getLogger(__name__)


class _InputType(qt.QWidget):
    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QHBoxLayout())
        self.layout().addWidget(qt.QLabel("input type:"))
        self._inputTypeCB = qt.QComboBox(parent=self)
        for input_type in InputType.values():
            self._inputTypeCB.addItem(input_type)
        self.layout().addWidget(self._inputTypeCB)

        # expose API
        self.currentChanged = self._inputTypeCB.currentIndexChanged

    def getInputType(self):
        """Return the current input type"""
        return InputType.from_value(self._inputTypeCB.currentText())

    def setInputType(self, input_type):
        _input_type = InputType.from_value(input_type)
        idx = self._inputTypeCB.findText(_input_type.value)
        assert idx >= 0
        self._inputTypeCB.setCurrentIndex(idx)


class XASObjectWindow(qt.QMainWindow):
    def __init__(self, parent):
        qt.QMainWindow.__init__(self, parent)
        self.setWindowFlags(qt.Qt.Widget)
        self._mainWindow = XASObjectDialog(self)
        self.setCentralWidget(self._mainWindow)

        self._plotDW = qt.QDockWidget(self)
        self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._plotDW)
        self._plotDW.setFeatures(qt.QDockWidget.DockWidgetMovable)
        self._plot = _ToggleableSpectrumPlot(self)
        self._plotDW.setWidget(self._plot)

        # connect signal / slot
        self._mainWindow.editingFinished.connect(self.loadXasObject)

    def getMainWindow(self):
        return self._mainWindow

    def setDatFile(self, file_path):
        self._mainWindow.setDatFile(file_path=file_path)

    def setXmuFile(self, file_path):
        self._mainWindow.setXmuFile(file_path=file_path)

    def setCsvFile(self, file_path):
        self._mainWindow.setCsvFile(file_path=file_path)

    def setEnergyColName(self, name):
        self._mainWindow.setEnergyColName(name)

    def setAbsColName(self, name):
        self._mainWindow.setAbsColName(name)

    def setMonitorColName(self, name):
        self._mainWindow.setMonitorColName(name)

    def setScanTitle(self, name):
        self._mainWindow.setScanTitle(name)

    def setCurrentType(self, input_type):
        self._mainWindow.setCurrentType(input_type=input_type)

    def setSpectraUrl(self, url):
        self._mainWindow.setSpectraUrl(url=url)

    def setEnergyUrl(self, url):
        self._mainWindow.setEnergyUrl(url=url)

    def setEnergyUnit(self, unit):
        self._mainWindow.setEnergyUnit(unit=unit)

    def setDimensions(self, dims):
        self._mainWindow.setDimensions(dims=dims)

    def setConfigurationUrl(self, url):
        self._mainWindow.setConfigurationUrl(url)

    def getConfigurationUrl(self):
        return self._mainWindow.getConfigurationUrl()

    def getAdvanceHdf5Information(self):
        return self._mainWindow.getAdvanceHdf5Information()

    def loadXasObject(self):
        """Load XasObject from information contained in the GUI
        and update plot"""
        self._plot.clear()
        try:
            xas_obj = self._mainWindow.buildXASObject()
        except Exception as e:
            _logger.info(str(e))
        else:
            if xas_obj is not None:
                self._plot.setXasObject(xas_obj=xas_obj)

    def buildXASObject(self):
        return self._mainWindow.buildXASObject()


class _ToggleableSpectrumPlot(qt.QWidget):
    _BUTTON_ICON = qt.QStyle.SP_ToolBarVerticalExtensionButton

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QGridLayout())
        self._toggleButton = qt.QPushButton(self)
        self.layout().addWidget(self._toggleButton, 0, 1, 1, 1)

        self._plot = _SpectrumPlot(parent=self)
        self.layout().addWidget(self._plot, 1, 0, 1, 2)
        # set up
        self._setButtonIcon(show=True)

        # Signal / slot connection
        self._toggleButton.clicked.connect(self.toggleSpectrumPlot)

    def toggleSpectrumPlot(self):
        visible = not self._plot.isVisible()
        self._setButtonIcon(show=visible)
        self._plot.setVisible(visible)

    def _setButtonIcon(self, show):
        style = qt.QApplication.instance().style()
        # return a QIcon
        icon = style.standardIcon(self._BUTTON_ICON)
        if show is False:
            pixmap = icon.pixmap(32, 32).transformed(qt.QTransform().scale(1, -1))
            icon = qt.QIcon(pixmap)
        self._toggleButton.setIcon(icon)

    def setXasObject(self, xas_obj):
        self._plot.setXasObject(xas_obj=xas_obj)

    def clear(self):
        self._plot.clear()


class _SpectrumPlot(qt.QWidget):
    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QHBoxLayout())
        from est.gui.XasObjectViewer import SpectrumViewer

        self._plot = SpectrumViewer(self)
        from est.gui.XasObjectViewer import _plot_spectrum

        self._plot.addCurveOperation(_plot_spectrum)
        self._plot.setWindowFlags(qt.Qt.Widget)
        # self._plot.setVisible(True)
        self.layout().addWidget(self._plot)

        # expose API
        self.addXMarker = self._plot._plotWidget.addXMarker

    def setXasObject(self, xas_obj):
        self._plot.setXasObject(xas_obj=xas_obj)

    def clear(self):
        self._plot.setXasObject(None)


class XASObjectDialog(qt.QWidget):
    """
    Interface used to select inputs for defining a XASObject
    """

    editingFinished = qt.Signal()

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QGridLayout())
        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
        self.layout().addWidget(spacer, 0, 0)
        self._inputType = _InputType(parent=self)
        self.layout().addWidget(self._inputType, 0, 1)

        # single spectrum (.csv, .xmu)
        self._singleSpectrumDialog = _XASObjectFrmSingleSpectrum(parent=self)
        self.layout().addWidget(self._singleSpectrumDialog, 1, 0, 1, 2)
        # .h5 file
        self._h5Dialog = _XASObjFrmH5(parent=self)
        self.layout().addWidget(self._h5Dialog, 2, 0, 1, 2)
        # spec file
        self._specDialog = _XASObjectFrmSpec(parent=self)
        self.layout().addWidget(self._specDialog, 3, 0, 1, 2)

        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer, 99, 0)

        # connect signal / slot
        self._inputType.currentChanged.connect(self._updateWidgetVisibility)
        self._inputType.currentChanged.connect(self._editingIsFinished)
        for diag in (self._singleSpectrumDialog, self._specDialog):
            diag._inputLe.editingFinished.connect(self._editingIsFinished)
        self._specDialog._dimensionWidget.sigInputChanged.connect(
            self._editingIsFinished
        )
        self._specDialog._scanTitleName.currentIndexChanged.connect(
            self._editingIsFinished
        )
        self._specDialog._inputLe.editingFinished.connect(self._updateSpecFile)
        self._h5Dialog.editingFinished.connect(self._editingIsFinished)

        # expose API
        self.getCurrentType = self._inputType.getInputType
        self.setCurrentType = self._inputType.setInputType
        self.setEnergyColName = self._specDialog.setEnergyColName
        self.getEnergyColName = self._specDialog.getEnergyColName
        self.setAbsColName = self._specDialog.setAbsColName
        self.getAbsColName = self._specDialog.getAbsColName
        self.setMonitorColName = self._specDialog.setMonitorColName
        self.getMonitorColName = self._specDialog.getMonitorColName
        self.setScanTitle = self._specDialog.setScanTitle
        self.getScanTitle = self._specDialog.getScanTitle
        self.setDatFile = self._specDialog.setFileSelected
        self.getDatFile = self._specDialog.getFileSelected
        self.setXmuFile = self._singleSpectrumDialog.setFileSelected
        self.setCsvFile = self._singleSpectrumDialog.setFileSelected
        self.setSpectraUrl = self._h5Dialog.setSpectraUrl
        self.getSpectraUrl = self._h5Dialog.getSpectraUrl
        self.setEnergyUrl = self._h5Dialog.setEnergyUrl
        self.getEnergyUrl = self._h5Dialog.getEnergyUrl
        self.setEnergyUnit = self._h5Dialog.setEnergyUnit
        self.getEnergyUnit = self._h5Dialog.getEnergyUnit
        self.setConfigurationUrl = self._h5Dialog.setConfigurationUrl
        self.getConfigurationUrl = self._h5Dialog.getConfigurationUrl
        self.getDimensions = self._h5Dialog.getDimensions
        self.setDimensions = self._h5Dialog.setDimensions

        # default setting
        self._updateWidgetVisibility()

    def getAdvanceHdf5Information(self):
        return self._h5Dialog.advanceInfo

    def _updateSpecFile(self, *args, **kwargs):
        self._specDialog._loadInfoFromSpecFile(self.getDatFile())

    def _updateWidgetVisibility(self):
        self._singleSpectrumDialog.setVisible(
            self._inputType.getInputType()
            in (InputType.xmu_spectrum, InputType.csv_spectrum)
        )
        self._h5Dialog.setVisible(
            self._inputType.getInputType() == InputType.hdf5_spectra
        )
        self._specDialog.setVisible(
            self._inputType.getInputType() == InputType.dat_spectrum
        )

    def buildXASObject(self):
        if self.getCurrentType() == InputType.dat_spectrum:
            specDiag = self._specDialog
            columns = specDiag.getColumnSelected()
            return read_frm_file(
                file_path=specDiag.getFileSelected(),
                energy_unit=specDiag.getEnergyUnit(),
                columns_names=columns,
            )
        elif self.getCurrentType() in (
            InputType.xmu_spectrum,
            InputType.csv_spectrum,
        ):
            spectrumDiag = self._singleSpectrumDialog
            return read_frm_file(
                file_path=spectrumDiag.getFileSelected(),
                energy_unit=spectrumDiag.getEnergyUnit(),
            )
        elif self.getCurrentType() == InputType.hdf5_spectra:
            spectra_url = self._h5Dialog.getSpectraUrl()
            energy_url = self._h5Dialog.getEnergyUrl()
            energy_unit = self._h5Dialog.getEnergyUnit()

            def check_url(url_path, name):
                if url_path in (None, ""):
                    raise ValueError(" ".join(("No", name, "url defined")))
                if isinstance(url_path, DataUrl):
                    url = url_path
                else:
                    url = DataUrl(path=url_path)
                if not url.is_valid():
                    raise ValueError(
                        " ".join(
                            (
                                name,
                                "url is invalid. Does the file / path still exists ?",
                            )
                        )
                    )

            check_url(spectra_url, "spectra")
            check_url(energy_url, "energy / channel")
            xas_obj = read_xas(
                spectra_url=self._h5Dialog.getSpectraUrl(),
                channel_url=self._h5Dialog.getEnergyUrl(),
                energy_unit=energy_unit,
                config_url=self._h5Dialog.getConfigurationUrl(),
                dimensions=self._h5Dialog.getDimensions(),
            )
            attr_names = "I0", "I1", "I2", "mu_ref"
            attr_values = (
                self._h5Dialog.advanceInfo.getI0Url(),
                self._h5Dialog.advanceInfo.getI1Url(),
                self._h5Dialog.advanceInfo.getI2Url(),
                self._h5Dialog.advanceInfo.getMuRefUrl(),
            )
            for attr_name, attr_value in zip(attr_names, attr_values):
                if attr_value is not None:
                    xas_obj.attach_3d_array(attr_name, attr_value)
            return xas_obj
        else:
            raise ValueError("unmanaged input type")

    def _editingIsFinished(self, *args, **kwargs):
        self.editingFinished.emit()


class _XASObjectFrmSingleSpectrum(qt.QWidget):
    """
    Interface used to define a XAS object from a single .dat file
    """

    class FileQLineEdit(qt.QLineEdit):
        """QLineEdit to handle a file path"""

        def dropEvent(self, event):
            if event.mimeData().hasFormat("text/uri-list"):
                for url in event.mimeData().urls():
                    self.setText(str(url.path()))

        def supportedDropActions(self):
            """Inherited method to redefine supported drop actions."""
            return qt.Qt.CopyAction | qt.Qt.MoveAction

        def dragEnterEvent(self, event):
            if event.mimeData().hasFormat("text/uri-list"):
                event.accept()
                event.setDropAction(qt.Qt.CopyAction)
            else:
                qt.QWidget.dragEnterEvent(self, event)

        def dragMoveEvent(self, event):
            if event.mimeData().hasFormat("text/uri-list"):
                event.setDropAction(qt.Qt.CopyAction)
                event.accept()
            else:
                qt.QWidget.dragMoveEvent(self, event)

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QGridLayout())

        # select file
        self.layout().addWidget(qt.QLabel("file", self), 0, 0, 1, 1)
        self._inputLe = self.FileQLineEdit("", self)
        self.layout().addWidget(self._inputLe, 0, 1, 1, 1)
        self._selectPB = qt.QPushButton("select", self)
        self.layout().addWidget(self._selectPB, 0, 2, 1, 1)

        # select energy
        self.layout().addWidget(qt.QLabel("energy unit", self), 1, 0, 1, 1)
        self._energyUnitSelector = EnergyUnitSelector(parent=self)
        self.layout().addWidget(self._energyUnitSelector, 1, 1, 1, 1)

        # spacer
        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer, 999, 0, 1, 1)

        # signal / slot connection
        self._selectPB.pressed.connect(self._selectFile)

    def getFileSelected(self):
        return self._inputLe.text()

    def _getNameFilters(self):
        return ["spectrum (*.xmu, *.csv)", "All Files (*)"]

    def _selectFile(self, *args, **kwargs):
        old = self.blockSignals(True)
        dialog = qt.QFileDialog(self)
        dialog.setFileMode(qt.QFileDialog.ExistingFile)
        dialog.setNameFilters(self._getNameFilters())

        if not dialog.exec_():
            dialog.close()
            return

        fileSelected = dialog.selectedFiles()
        if len(fileSelected) == 0:
            return
        else:
            assert len(fileSelected) == 1
            self.setFileSelected(fileSelected[0])

        self.blockSignals(old)

    def setFileSelected(self, file_path):
        self._inputLe.setText(file_path)

    def getEnergyUnit(self):
        return self._energyUnitSelector.getUnit()

    def setEnergyUnit(self, unit):
        return self._energyUnitSelector.setUnit(unit=unit)


class _XASObjectFrmSpec(_XASObjectFrmSingleSpectrum):
    def __init__(self, parent):
        super().__init__(parent)

        self.layout().addWidget(qt.QLabel("scan title"), 2, 0, 1, 1)
        self._scanTitleName = qt.QComboBox(self)
        self.layout().addWidget(self._scanTitleName, 2, 1, 1, 1)
        # dimension definition
        self._dimensionWidget = _SpecColNames(parent=self)
        self.layout().addWidget(self._dimensionWidget, 3, 0, 1, 2)

    def _getNameFilters(self):
        return ["spec files (*.dat)", "All Files (*)"]

    def setScanTitle(self, scan_title):
        index = self._scanTitleName.findText(scan_title)
        if index >= 0:
            self._scanTitleName.setCurrentIndex(index)

    def getScanTitle(self):
        return self._scanTitleName.currentText()

    def setEnergyColName(self, name):
        self._dimensionWidget.setEnergyColName(name)

    def getEnergyColName(self):
        return self._dimensionWidget.getEnergyColName()

    def setAbsColName(self, name):
        self._dimensionWidget.setAbsColName(name)

    def getAbsColName(self):
        return self._dimensionWidget.getAbsColName()

    def setMonitorColName(self, name):
        self._dimensionWidget.setMonitorColName(name)

    def getMonitorColName(self):
        return self._dimensionWidget.getMonitorColName()

    def getColumnSelected(self):
        col_names = self._dimensionWidget.getColumnSelected()
        col_names["scan_title"] = self.getScanTitle()
        return col_names

    def _loadInfoFromSpecFile(self, file_path):
        old = self.blockSignals(True)
        try:
            self._dimensionWidget.setFile(file_path)
            spec_file = SpecFile(file_path)
            titles = set()
            for scan in spec_file:
                titles.add(scan.scan_header_dict["S"])
            titles = sorted(titles)
            for title in titles:
                self._scanTitleName.addItem(title)
        except Exception as e:
            _logger.warning(e)
        self.blockSignals(old)

    def setFileSelected(self, file_path):
        super().setFileSelected(file_path=file_path)
        self._loadInfoFromSpecFile(file_path=file_path)


class _URLSelector(qt.QWidget):
    def __init__(self, parent, name, layout=None, position=None):
        qt.QWidget.__init__(self, parent)
        self.name = name
        if layout is None:
            layout = self.setLayout(qt.QGridLayout())
            position = (0, 0)
        layout.addWidget(qt.QLabel(name + ":", parent=self), position[0], position[1])
        self._qLineEdit = qt.QLineEdit("", parent=self)
        layout.addWidget(self._qLineEdit, position[0], position[1] + 1)
        self._qPushButton = qt.QPushButton("select", parent=self)
        layout.addWidget(self._qPushButton, position[0], position[1] + 2)

        # connect signal / slot
        self._qPushButton.clicked.connect(self._selectFile)

    def _selectFile(self, *args, **kwargs):
        dialog = DataFileDialog(self)

        if not dialog.exec_():
            dialog.close()
            return None

        if dialog.selectedUrl() is not None:
            self._qLineEdit.setText(dialog.selectedUrl())

    def getUrlPath(self):
        url = self._qLineEdit.text()
        if url == "":
            return None
        else:
            return DataUrl(path=url)

    def setUrlPath(self, url):
        if isinstance(url, DataUrl):
            url = url.path()
        self._qLineEdit.setText(url)


class _XASObjFrmH5(qt.QTabWidget):
    """
    Interface used to define a XAS object from h5 files and data path
    """

    editingFinished = qt.Signal()
    """signal emitted when edition is finished"""

    def __init__(self, parent=None):
        qt.QTabWidget.__init__(self, parent)
        # Mandatory information
        self._basicInformation = _MandatoryXASObjFrmH5(self)
        self.addTab(self._basicInformation, "basic information")
        # Optional information
        self._advanceInformation = _OptionalRefsXASObjFrmH5(self)
        self.addTab(self._advanceInformation, "advanced information")

        # connect signal / slot
        self._basicInformation.editingFinished.connect(self._editingIsFinished)
        self._advanceInformation.editingFinished.connect(self._editingIsFinished)

        # expose API
        self.setDimensions = self._basicInformation.setDimensions
        self.getDimensions = self._basicInformation.getDimensions
        self.setConfigurationUrl = self._basicInformation.setConfigurationUrl
        self.getConfigurationUrl = self._basicInformation.getConfigurationUrl
        self.setEnergyUrl = self._basicInformation.setEnergyUrl
        self.getEnergyUrl = self._basicInformation.getEnergyUrl
        self.setSpectraUrl = self._basicInformation.setSpectraUrl
        self.getSpectraUrl = self._basicInformation.getSpectraUrl
        self.setEnergyUnit = self._basicInformation.setEnergyUnit
        self.getEnergyUnit = self._basicInformation.getEnergyUnit

    @property
    def advanceInfo(self):
        return self._advanceInformation

    def _editingIsFinished(self, *args, **kwargs):
        self.editingFinished.emit()


class _OptionalRefsXASObjFrmH5(qt.QWidget):
    """Widget containing optional information that we can associate to
    spectrum.
    """

    editingFinished = qt.Signal()
    """signal emitted when edition is finished"""

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QGridLayout())
        # I0
        self._I0Selector = _URLSelector(
            parent=self, name="I0 url", layout=self.layout(), position=(0, 0)
        )
        # I1
        self._I1Selector = _URLSelector(
            parent=self, name="I1 url", layout=self.layout(), position=(1, 0)
        )
        # I2
        self._I2Selector = _URLSelector(
            parent=self, name="I2 url", layout=self.layout(), position=(2, 0)
        )
        # mu ref
        self._muRefSelector = _URLSelector(
            parent=self,
            name="{} ref".format(MU_CHAR),
            layout=self.layout(),
            position=(3, 0),
        )

        # connect signal / slot
        self._I0Selector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._I1Selector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._I2Selector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._muRefSelector._qLineEdit.textChanged.connect(self._editingIsFinished)

    def _editingIsFinished(self, *args, **kwargs):
        self.editingFinished.emit()

    def getI0Url(self):
        return self._I0Selector.getUrlPath()

    def setI0Url(self, url):
        self._I0Selector.setUrlPath(url)

    def getI1Url(self):
        return self._I1Selector.getUrlPath()

    def setI1Url(self, url):
        self._I1Selector.setUrlPath(url)

    def getI2Url(self):
        return self._I2Selector.getUrlPath()

    def setI2Url(self, url):
        self._I2Selector.setUrlPath(url)

    def getMuRefUrl(self):
        return self._muRefSelector.getUrlPath()

    def setMuRefUrl(self, url):
        self._muRefSelector.setUrlPath(url)


class _MandatoryXASObjFrmH5(qt.QWidget):
    """Widget containing mandatory information"""

    editingFinished = qt.Signal()
    """signal emitted when edition is finished"""

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QGridLayout())
        # spectra url
        self._spectraSelector = _URLSelector(
            parent=self, name="spectra url", layout=self.layout(), position=(0, 0)
        )

        self._bufWidget = qt.QWidget(parent=self)
        self._bufWidget.setLayout(qt.QHBoxLayout())
        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
        self._bufWidget.layout().addWidget(spacer)
        self._dimensionSelection = _SpectraDimensions(parent=self._bufWidget)
        self._bufWidget.layout().addWidget(self._dimensionSelection)
        self.layout().addWidget(self._bufWidget, 1, 1)

        # energy / channel url
        self._energySelector = _URLSelector(
            parent=self,
            name="energy /channel url",
            layout=self.layout(),
            position=(2, 0),
        )
        self._energyUnit = EnergyUnitSelector(parent=self)
        self.layout().addWidget(self._energyUnit, 2, 3, 1, 1)
        # configuration url
        self._configSelector = _URLSelector(
            parent=self, name="configuration url", layout=self.layout(), position=(3, 0)
        )

        # connect signal / slot
        self._spectraSelector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._energySelector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._configSelector._qLineEdit.textChanged.connect(self._editingIsFinished)
        self._dimensionSelection.sigDimensionChanged.connect(self._editingIsFinished)
        self._energyUnit.currentIndexChanged.connect(self._editingIsFinished)

        # expose APi
        self.setDimensions = self._dimensionSelection.setDimensions
        self.getDimensions = self._dimensionSelection.getDimensions

    def getSpectraUrl(self):
        """

        :return: the DataUrl of the spectra
        :rtype: DataUrl
        """
        return self._spectraSelector.getUrlPath()

    def getEnergyUrl(self):
        """

        :return: the DataUrl of energy / channel
        :rtype: DataUrl
        """
        return self._energySelector.getUrlPath()

    def getConfigurationUrl(self):
        """

        :return: the DataUrl of the configuration
        :rtype: DataUrl
        """
        return self._configSelector.getUrlPath()

    def setSpectraUrl(self, url):
        self._spectraSelector.setUrlPath(url)

    def setEnergyUrl(self, url):
        self._energySelector.setUrlPath(url)

    def setConfigurationUrl(self, url):
        self._configSelector.setUrlPath(url)

    def _editingIsFinished(self, *args, **kwargs):
        self.editingFinished.emit()

    def getDimensionsInfo(self):
        """

        :return: return the information regarding each dimensions
                 (dim0, dim1, dim2)
        :rtype: tuple
        """
        return self._dimensionSelection.getDimensions()

    def getEnergyUnit(self):
        return self._energyUnit.getUnit()

    def setEnergyUnit(self, unit):
        self._energyUnit.setUnit(unit=unit)


class _QColumnComboBox(qt.QComboBox):
    def __init__(self, parent):
        qt.QComboBox.__init__(self, parent)

    def setColumnsNames(self, colums_names: Iterable):
        for column_name in colums_names:
            self.addItem(column_name)

    def setColumnName(self, name):
        index = self.findText(name)
        if index >= 0:
            self.setCurrentIndex(index)
        else:
            _logger.info("Unable to find name: {}".format(name))

    def getColumnName(self):
        return self.currentText()


class _QDimComboBox(qt.QComboBox):
    def __init__(self, parent):
        qt.QComboBox.__init__(self, parent)
        self.addItem(Dim.DIM_0.value)
        self.addItem(Dim.DIM_1.value)
        self.addItem(Dim.DIM_2.value)

    def setDim(self, dim):
        dim = Dim.from_value(dim)
        assert dim in (Dim.DIM_0, Dim.DIM_1, Dim.DIM_2)
        index = self.findText(dim.value)
        assert index >= 0
        self.setCurrentIndex(index)

    def getDim(self):
        return Dim.from_value(self.currentText())


class _SpectraDimensions(qt.QWidget):
    sigDimensionChanged = qt.Signal()
    """Signal emitted when dimension change"""

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QFormLayout())
        self._dim0 = _QDimComboBox(parent=self)
        self.layout().addRow("dim 0", self._dim0)
        self._dim1 = _QDimComboBox(parent=self)
        self.layout().addRow("dim 1", self._dim1)
        self._dim2 = _QDimComboBox(parent=self)
        self.layout().addRow("dim 2", self._dim2)

        # set up
        self._dim0.setDim(Dim.DIM_2)
        self._dim1.setDim(Dim.DIM_1)
        self._dim2.setDim(Dim.DIM_0)

        # connect Signal / Slot
        self._dim0.currentTextChanged.connect(self._insureDimUnicity)
        self._dim1.currentTextChanged.connect(self._insureDimUnicity)
        self._dim2.currentTextChanged.connect(self._insureDimUnicity)

    def _insureDimUnicity(self):
        last_modified = self.sender()
        if last_modified is self._dim0:
            get_second, get_third = self._dim1, self._dim2
        elif last_modified is self._dim1:
            get_second, get_third = self._dim0, self._dim2
        elif last_modified is self._dim2:
            get_second, get_third = self._dim0, self._dim1
        else:
            raise RuntimeError("Sender should be in dim0, dim1, dim2")

        assert last_modified != get_second
        assert last_modified != get_third
        assert type(last_modified) == type(get_second) == type(get_third)
        value_set = {Dim.DIM_2, Dim.DIM_0, Dim.DIM_1}
        last_value_set = last_modified.getDim()
        value_set.remove(last_value_set)

        old_1 = last_modified.blockSignals(True)
        old_2 = get_second.blockSignals(True)
        old_3 = get_third.blockSignals(True)
        if get_second.getDim() in value_set:
            value_set.remove(get_second.getDim())
            get_third.setDim(value_set.pop())
        elif get_third.getDim() in value_set:
            value_set.remove(get_third.getDim())
            get_second.setDim(value_set.pop())
        else:
            get_second.setDim(value_set.pop())
            get_third.setDim(value_set.pop())

        last_modified.blockSignals(old_1)
        get_second.blockSignals(old_2)
        get_third.blockSignals(old_3)
        self.sigDimensionChanged.emit()

    def getDimensions(self):
        """

        :return: return the information regarding each dimensions
                 (dim0, dim1, dim2)
        :rtype: tuple
        """
        return (
            self._dim0.currentText(),
            self._dim1.currentText(),
            self._dim2.currentText(),
        )

    def setDimensions(self, dims):
        """

        :param dims: tuple containing (dim0, dim1, dim2)
        :type: tuple
        """
        assert isinstance(dims, tuple)
        assert len(dims) == 3
        self._dim0.setDim(dims[0])
        self._dim1.setDim(dims[1])
        self._dim2.setDim(dims[2])


class _SpecColNames(qt.QWidget):
    sigInputChanged = qt.Signal()
    """Signal emitted when dimension change"""

    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QFormLayout())
        self._energyColNamCB = _QColumnComboBox(parent=self)
        self.layout().addRow("energy col name", self._energyColNamCB)
        self._absColNamCB = _QColumnComboBox(parent=self)
        self.layout().addRow("absorption col name", self._absColNamCB)
        self._monitorColNamCB = _QColumnComboBox(parent=self)
        self._monitorColNamCB.setEnabled(False)
        self._useMonitorCB = qt.QCheckBox("monitor col name")
        self.layout().addRow(self._useMonitorCB, self._monitorColNamCB)

        # connect Signal / Slot
        self._energyColNamCB.currentIndexChanged.connect(self._propagateSigChanged)
        self._absColNamCB.currentIndexChanged.connect(self._propagateSigChanged)
        self._monitorColNamCB.currentIndexChanged.connect(self._propagateSigChanged)
        self._useMonitorCB.toggled.connect(self._monitorColNamCB.setEnabled)
        self._useMonitorCB.toggled.connect(self._propagateSigChanged)

    def _propagateSigChanged(self):
        self.sigInputChanged.emit()

    def getColumnSelected(self) -> dict:
        """

        :return: return the information regarding each energy and mu
        :rtype: dict
        """
        return {
            "energy": self._energyColNamCB.currentText(),
            "mu": self._absColNamCB.currentText(),
            "monitor": self.getMonitorColName(),
        }

    def getMonitorColName(self):
        if self.useMonitor():
            return self._monitorColNamCB.currentText()
        else:
            return None

    def useMonitor(self):
        return self._useMonitorCB.isChecked()

    def setEnergyColName(self, name):
        self._energyColNamCB.setColumnName(name)

    def getEnergyColName(self):
        return self._energyColNamCB.getColumnName()

    def setAbsColName(self, name):
        self._absColNamCB.setColumnName(name)
        if isinstance(name, str):
            self._useMonitorCB.setChecked(True)

    def getAbsColName(self):
        return self._absColNamCB.getColumnName()

    def setMonitorColName(self, name):
        self._monitorColNamCB.setColumnName(name)

    def setFile(self, file_path):
        old = self.blockSignals(True)
        prev_energy = self._energyColNamCB.currentText()
        prev_absorption = self._absColNamCB.currentText()
        prev_monitor = self._monitorColNamCB.currentText()
        self._energyColNamCB.clear()
        self._absColNamCB.clear()
        self._monitorColNamCB.clear()

        spec_file = SpecFile(file_path)
        col_names = set()
        for scan in spec_file:
            col_names.update(scan.labels)
        # sort column name
        col_names = sorted(col_names)
        self._energyColNamCB.setColumnsNames(col_names)
        self._absColNamCB.setColumnsNames(col_names)
        self._monitorColNamCB.setColumnsNames(col_names)
        self._energyColNamCB.setColumnName(prev_energy)
        self._absColNamCB.setColumnName(prev_absorption)
        self._monitorColNamCB.setColumnName(prev_monitor)
        self.blockSignals(old)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "08/08/2019"


from Orange.widgets.widget import OWWidget
from Orange.widgets.widget import Input, Output
from est.core.types import XASObject
from orangecontrib.est.utils import Converter
import Orange.data


class ConverterOW(OWWidget):
    """
    Offer a conversion from XASObject to Orange.data.Table, commonly used
    from Orange widget
    """

    name = "converter xas_obj -> Table"
    id = "orange.widgets.xas.utils.converter"
    description = "convert a XASObject to a Orange.data.Table"
    icon = "icons/converter.png"
    priority = 5
    category = "esrfWidgets"
    keywords = ["spectroscopy", "signal", "output", "file"]

    want_main_area = False
    resizing_enabled = False
    allows_cycle = False

    class Inputs:
        xas_obj = Input("xas_obj", XASObject, default=True)
        # simple compatibility for some Orange widget and especialy the
        # 'spectroscopy add-on'

    class Outputs:
        res_data_table = Output("Data", Orange.data.Table)
        # by default we want to avoid sending 'Orange.data.Table' to avoid
        # loosing the XASObject flow process and results.

    @Inputs.xas_obj
    def process(self, xas_object):
        if xas_object is None:
            return
        data_table = Converter.toDataTable(xas_object=xas_object)
        self.Outputs.res_data_table.send(data_table)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "07/08/2019"


from silx.gui import qt
from est.gui.larch.utils import _OptionalQDoubleSpinBox


class _MBackParameters(qt.QWidget):
    """
    Widget for setting the configuration of the larch 'mback' process
    """

    sigChanged = qt.Signal()
    """Signal emitted when parameters changed"""

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QFormLayout())

        # z
        self._atomicNumberSB = qt.QSpinBox(parent=self)
        self._atomicNumberSB.setToolTip("atomic number of the absorber")
        self._atomicNumberSB.setMinimum(1)
        self._atomicNumberSB.setValue(13)
        self._atomicNumberSB.setMaximum(118)
        self.layout().addRow(qt.QLabel("z", parent=self), self._atomicNumberSB)
        # edge
        self._edgeCB = qt.QComboBox(parent=self)
        for val in ("K", "L1", "L2", "L3", "M1", "M2", "M3", "M4", "M5"):
            self._edgeCB.addItem(val)
        index = self._edgeCB.findText("K")
        assert index >= 0
        self._edgeCB.setCurrentIndex(index)
        self._edgeCB.setToolTip("x-ray absorption edge")
        self.layout().addRow(qt.QLabel("edge", parent=self), self._edgeCB)
        # e0
        self._e0SB = _OptionalQDoubleSpinBox(parent=self)
        self._e0SB.setToolTip(
            "edge energy, in eV.  If None, it will be " "determined here."
        )
        self._e0SB.setMinimum(0.0)
        self._e0SB.setValue(2000.0)
        self._e0SB.setValue(None)
        self.layout().addRow(qt.QLabel("e0", parent=self), self._e0SB)
        # pre1
        self._pre1SB = _OptionalQDoubleSpinBox(parent=self)
        self._pre1SB.setToolTip("low E range (relative to e0) for pre-edge " "region")
        self._pre1SB.setValue(None)
        self._pre1SB.setContentsMargins(0, 0, 0, 0)
        self.layout().addRow(qt.QLabel("pre1", parent=self), self._pre1SB)
        # pre2
        self._pre2SB = _OptionalQDoubleSpinBox(parent=self)
        self._pre2SB.setToolTip("high E range (relative to e0) for pre-edge " "region")
        self._pre2SB.setValue(-50)
        self._pre2SB.setContentsMargins(0, 0, 0, 0)
        self.layout().addRow(qt.QLabel("pre2", parent=self), self._pre2SB)
        # norm1
        self._norm1SB = _OptionalQDoubleSpinBox(parent=self)
        self._norm1SB.setToolTip("low E range (relative to e0) for post-edge " "region")
        self._norm1SB.setContentsMargins(0, 0, 0, 0)
        self._norm1SB.setValue(100)
        self.layout().addRow(qt.QLabel("norm1", parent=self), self._norm1SB)
        # norm2
        self._norm2SB = _OptionalQDoubleSpinBox(parent=self)
        self._norm2SB.setToolTip(
            "high E range (relative to e0) for post-edge " "region"
        )
        self._norm2SB.setValue(None)
        self._norm2SB.setContentsMargins(0, 0, 0, 0)
        self.layout().addRow(qt.QLabel("norm2", parent=self), self._norm2SB)
        # order
        self._norm2SB.setToolTip(
            "order of the legendre polynomial for " "normalization"
        )
        self._orderSB = qt.QSpinBox(parent=self)
        self.layout().addRow(qt.QLabel("order", parent=self), self._orderSB)
        self._orderSB.setRange(0, 5)
        self._orderSB.setValue(3)
        # leexiang
        self._leexiangCB = qt.QCheckBox("", parent=self)
        self._leexiangCB.setToolTip(
            "boolean (default False)  to use the Lee & " "Xiang extension"
        )
        self.layout().addRow(qt.QLabel("leexiang", parent=self), self._leexiangCB)
        # tables
        self._tableCB = qt.QComboBox(parent=self)
        self._tableCB.setToolTip(
            'tabulated scattering factors: "chantler" '
            '(default) or "cl" (cromer-liberman)'
        )
        for item in ("chantler", "cl"):
            self._tableCB.addItem(item)
        index = self._tableCB.findText("chantler")
        assert index >= 0
        self._tableCB.setCurrentIndex(index)
        self.layout().addRow(qt.QLabel("tables", parent=self), self._tableCB)
        # fit erfc
        self._fiterfc = qt.QCheckBox(parent=self)
        self._fiterfc.setToolTip("fit parameters of error function " "(default False)")
        self.layout().addRow(qt.QLabel("fit erfc", parent=self), self._fiterfc)

        # expose API
        self.getPre1 = self._pre1SB.getValue
        self.setPre1 = self._pre1SB.setValue
        self.getPre2 = self._pre2SB.getValue
        self.setPre2 = self._pre2SB.setValue
        self.getNorm1 = self._norm1SB.getValue
        self.setNorm1 = self._norm1SB.setValue
        self.getNorm2 = self._norm2SB.getValue
        self.setNorm2 = self._norm2SB.setValue
        self.isUsingLeexiangExt = self._leexiangCB.isChecked
        self.setUsingLeexiangExt = self._leexiangCB.setChecked
        self.isFitParamErrFunc = self._fiterfc.isChecked
        self.setFitParamErrFunc = self._fiterfc.setChecked

        # connect signal / Slot
        self._atomicNumberSB.valueChanged.connect(self._valueChanged)
        self._edgeCB.currentIndexChanged.connect(self._valueChanged)
        self._e0SB.sigChanged.connect(self._valueChanged)
        self._pre1SB.sigChanged.connect(self._valueChanged)
        self._pre2SB.sigChanged.connect(self._valueChanged)
        self._norm1SB.sigChanged.connect(self._valueChanged)
        self._norm2SB.sigChanged.connect(self._valueChanged)
        self._orderSB.valueChanged.connect(self._valueChanged)
        self._leexiangCB.toggled.connect(self._valueChanged)
        self._tableCB.currentIndexChanged.connect(self._valueChanged)
        self._fiterfc.toggled.connect(self._valueChanged)

        # avoid several emission of the sigChanged signal
        self._lastParameters = None

    def _valueChanged(self, *arg, **kwargs):
        currentParameters = self.getParameters()
        if currentParameters != self._lastParameters:
            self._lastParameters = currentParameters
            self.sigChanged.emit()

    def getParameters(self):
        return {
            "z": self.getAtomicNumber(),
            "edge": self.getEdge(),
            "e0": self.getE0(),
            "pre1": self.getPre1(),
            "pre2": self.getPre2(),
            "norm1": self.getNorm1(),
            "norm2": self.getNorm2(),
            "order": self.getOrder(),
            "leexiang": self.isUsingLeexiangExt(),
            "tables": self.getTables(),
            "fit_erfc": self.isFitParamErrFunc(),
        }

    def setParameters(self, parameters):
        assert isinstance(parameters, dict)
        for key, value in parameters.items():
            if key == "z":
                self.setAtomicNumber(value)
            elif key == "edge":
                self.setEdge(value)
            elif key == "e0":
                self.setE0(value)
            elif key == "pre1":
                self.setPre1(value)
            elif key == "pre2":
                self.setPre2(value)
            elif key == "norm1":
                self.setNorm1(value)
            elif key == "norm2":
                self.setNorm2(value)
            elif key == "order":
                self.setOrder(value)
            elif key == "leexiang":
                self.setUsingLeexiangExt(value)
            elif key == "tables":
                self.setTables(value)
            elif key == "fit_erfc":
                self.setFitParamErrFunc(value)

    def getAtomicNumber(self):
        return self._atomicNumberSB.value()

    def setAtomicNumber(self, value):
        return self._atomicNumberSB.setValue(value)

    def getEdge(self):
        return self._edgeCB.currentText()

    def setEdge(self, edge):
        index = self._edgeCB.findText(edge)
        assert index >= 0
        self._edgeCB.setCurrentIndex(index)

    def getE0(self):
        return self._e0SB.getValue()

    def setE0(self, value):
        self._e0SB.setValue(value=value)

    def getOrder(self):
        return self._orderSB.value()

    def setOrder(self, order):
        self._orderSB.setValue(order)

    def getTables(self):
        return self._tableCB.currentText()

    def setTables(self, tables):
        index = self._tableCB.findText(tables)
        if index >= 0:
            self._tableCB.setCurrentIndex(index)


if __name__ == "__main__":
    app = qt.QApplication([])
    widget = _MBackParameters()
    widget.show()
    app.exec_()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/07/2019"


import functools
import logging

from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.widgets.widget import Input, Output
import Orange.data
from PyMca5.PyMcaGui.physics.xas.XASNormalizationParameters import (
    XASNormalizationParameters,
)
from silx.gui import qt
from silx.gui.plot import LegendSelector
from est.gui.XasObjectViewer import (
    _plot_edge,
    _plot_norm,
    _plot_post_edge,
    _plot_pre_edge,
)
from orangecontrib.est.process import _ProcessForOrangeMixIn
from orangecontrib.est.process import ProcessRunnable
from orangecontrib.est.widgets.container import _ParameterWindowContainer
from orangecontrib.est.progress import QProgress
from orangecontrib.est.utils import Converter
from est.gui.e0calculator import E0CalculatorDialog
from est.core.types import XASObject
from est.gui.XasObjectViewer import XasObjectViewer, ViewType
import est.core.process.pymca.normalization

_logger = logging.getLogger(__file__)


class _XASNormalizationParametersPatched(XASNormalizationParameters):
    """This class will try to patch the XASNormalizationParameters with the
    E0Calculation widget"""

    sigE0CalculationRequested = qt.Signal()
    """Signal emitted when E0 computation is required"""

    def __init__(self, *args, **kwargs):
        XASNormalizationParameters.__init__(self, *args, **kwargs)
        # add E0CalculationWidget if can
        try:
            self.__addE0CalculationDialog()
        except Exception as e:
            _logger.warning("Fail to add the E0CalculationDialog. Reason is", str(e))
        else:
            self._e0CalcPB.pressed.connect(self._launchE0Calculator)

    def setE0(self, e0):
        e0_min = self.e0SpinBox.minimum()
        e0_max = self.e0SpinBox.maximum()
        if not (e0_min <= e0 <= e0_max):
            _logger.warning(
                "given e0 value (%s) is invalid, value should be "
                "between %s and %s" % (e0, e0_min, e0_max)
            )

        params = self.getParameters()
        params["E0Value"] = e0
        params["E0Method"] = "Manual"
        self.setParameters(ddict=params, signal=False)

    def __addE0CalculationDialog(self):
        # check we know where to set the button
        for attr in ("e0SpinBox", "jumpLine", "e0CheckBox"):
            if not hasattr(self, attr):
                raise NameError(
                    "%s not defined - pymca version not " "recognized" % attr
                )
        for widget, widget_index in zip((self.e0SpinBox, self.jumpLine), (3, 5)):
            if self.layout().indexOf(widget) != widget_index:
                raise ValueError("XASNormalizationParameters layout is not recognized.")

        style = qt.QApplication.instance().style()
        icon = style.standardIcon(qt.QStyle.SP_FileDialogContentsView)
        self._e0CalcPB = qt.QPushButton(icon, "", self)
        self.layout().addWidget(self._e0CalcPB, 1, 2)

    def _launchE0Calculator(self, *args, **kwargs):
        self.sigE0CalculationRequested.emit()


class NormalizationWindow(qt.QMainWindow):
    """Widget embedding the pymca parameter window and the display of the
    data currently process"""

    sigE0CalculationRequested = qt.Signal()
    """Signal emitted when E0 computation is required"""

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        # xas object viewer
        mapKeys = ["mu", "NormalizedMu", "NormalizedSignal", "NormalizedBackground"]
        self.xasObjViewer = XasObjectViewer(mapKeys=mapKeys)
        self.xasObjViewer._spectrumViews[0]._plotWidget.getXAxis().setLabel(
            "Energy (eV)"
        )
        self.xasObjViewer._spectrumViews[0]._plotWidget.getYAxis().setLabel(
            "Absorption (a.u.)"
        )
        self.setCentralWidget(self.xasObjViewer)
        self._pymcaWindow = _ParameterWindowContainer(
            parent=self, parametersWindow=_XASNormalizationParametersPatched
        )
        dockWidget = qt.QDockWidget(parent=self)

        # pymca window
        dockWidget.setWidget(self._pymcaWindow)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, dockWidget)
        dockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # legend selector
        self.legendDockWidget = LegendSelector.LegendsDockWidget(
            parent=self, plot=self.xasObjViewer._spectrumViews[0]._plotWidget
        )
        self.legendDockWidget.setAllowedAreas(
            qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea
        )
        self.legendDockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self.legendDockWidget)

        # volume key selection
        self.addDockWidget(
            qt.Qt.RightDockWidgetArea, self.xasObjViewer._mapView.keySelectionDocker
        )

        # plot settings
        for ope in (_plot_edge, _plot_norm, _plot_post_edge, _plot_pre_edge):
            self.xasObjViewer._spectrumViews[0].addCurveOperation(ope)

        self.setWindowFlags(qt.Qt.Widget)

        # expose API
        self.setE0 = self._pymcaWindow._mainwidget.setE0

        # connect signal / slot
        self.xasObjViewer.viewTypeChanged.connect(self._updateLegendView)

        # set up
        self._updateLegendView()

    def getNCurves(self):
        return len(self.xasObjViewer._spectrumViews._plot.getAllCurves())

    def _updateLegendView(self):
        index, viewType = self.xasObjViewer.getViewType()
        self.legendDockWidget.setVisible(viewType is ViewType.spectrum)
        self.xasObjViewer._mapView.keySelectionDocker.setVisible(
            viewType is ViewType.map
        )


class NormalizationOW(_ProcessForOrangeMixIn, OWWidget):
    """
    Widget used for signal extraction
    """

    name = "normalization"
    id = "orange.widgets.est.pymca.normalization"
    description = "Progress spectra normalization"
    icon = "icons/normalization.png"
    priority = 1
    category = "esrfWidgets"
    keywords = ["spectroscopy", "normalization"]

    want_main_area = True
    resizing_enabled = True
    allows_cycle = False

    process_function = est.core.process.pymca.normalization.PyMca_normalization

    _pymcaSettings = Setting(dict())
    """Store the configuration of the PyMca XASClass"""

    class Inputs:
        xas_obj = Input("xas_obj", XASObject, default=True)
        # simple compatibility for some Orange widget and especialy the
        # 'spectroscopy add-on'
        data_table = Input("Data", Orange.data.Table)

    class Outputs:
        res_xas_obj = Output("xas_obj", XASObject)
        # by default we want to avoid sending 'Orange.data.Table' to avoid
        # loosing the XASObject flow process and results.

    def __init__(self):
        super().__init__()
        self._latest_xas_obj = None
        self._window = NormalizationWindow(parent=self)
        layout = gui.vBox(self.mainArea, "normalization").layout()
        layout.addWidget(self._window)
        self._window.xasObjViewer.setWindowTitle("spectra")

        # expose API
        self.setE0 = self._window.setE0

        # manage settings
        if self._pymcaSettings != dict():
            self._window._pymcaWindow.setParameters(self._pymcaSettings)

        # connect signals / slots
        pymcaWindowContainer = self._window._pymcaWindow
        _sig = pymcaWindowContainer.sigChanged.connect(self._updateProcess)
        if hasattr(pymcaWindowContainer._mainwidget, "sigE0CalculationRequested"):
            pymcaWindowContainer._mainwidget.sigE0CalculationRequested.connect(
                self._getE0FrmDialog
            )

    def _updateProcess(self):
        self._update_settings()
        if self._latest_xas_obj:
            self.process(self._latest_xas_obj)

    def _update_settings(self):
        self._pymcaSettings = self._window._pymcaWindow.getParameters()

    @Inputs.data_table
    def processFrmDataTable(self, data_table):
        if data_table is None:
            return
        self.process(Converter.toXASObject(data_table=data_table))

    @Inputs.xas_obj
    def process(self, xas_obj):
        if xas_obj is None:
            return
        if not self._canProcess():
            _logger.warning(
                "There is some processing on going already, will"
                "not process the new dataset"
            )

        self._latest_xas_obj = xas_obj
        # update E0 if necessary
        if "e0" in xas_obj.configuration:
            self.setE0(xas_obj.configuration["e0"])
        self._startProcess()

        # setup the normalization process
        process_obj = QPyMca_normalization()
        process_obj._advancement.sigProgress.connect(self._setProgressValue)
        process_obj.set_properties(
            {"_pymcaSettings": self._window._pymcaWindow.getParameters()}
        )

        # update the processing thread
        thread = self.getProcessingThread()
        thread.init(process_obj=process_obj, xas_obj=self._latest_xas_obj)
        self._callback_finish = functools.partial(
            self._endProcess, self._latest_xas_obj
        )
        thread.finished.connect(self._callback_finish)
        # start processing
        thread.start(priority=qt.QThread.LowPriority)

    def _getE0FrmDialog(self):
        """Pop up an instance of E0CalculationDialog and get E0 from it"""
        if self._latest_xas_obj is None:
            return
        dialog = E0CalculatorDialog(xas_obj=self._latest_xas_obj, parent=None)
        if dialog.exec_():
            self.setE0(dialog.getE0())


class QPyMca_normalization(est.core.process.pymca.normalization.PyMca_normalization):
    """
    Normalization able to give advancement using qt.Signal and QThreadPool
    """

    def __init__(self):
        est.core.process.pymca.normalization.PyMca_normalization.__init__(self)
        self._advancement = QProgress("normalization")

    def _pool_process(self, xas_obj):
        self.pool = qt.QThreadPool()
        self.pool.setMaxThreadCount(5)
        for spectrum in xas_obj.spectra:
            runnable = ProcessRunnable(
                fct=est.core.process.pymca.normalization.process_spectr_norm,
                spectrum=spectrum,
                configuration=xas_obj.configuration,
                callback=self._advancement.increaseAdvancement,
            )
            self.pool.start(runnable)
        self.pool.waitForDone()


if __name__ == "__main__":
    from est.core.utils import spectra as spectra_utils
    from est.core.types import XASObject

    def create_dataset():
        dim1, dim2 = 20, 40

        energy, spectra = spectra_utils.create_dataset(
            shape=(256, dim1, dim2), noise=True
        )
        spectra = spectra * 1000
        return XASObject(energy=energy, spectra=spectra, dim1=dim1, dim2=dim2)

    xas_object = create_dataset()
    app = qt.QApplication([])
    widget = NormalizationOW()
    widget.process(xas_object)
    widget.show()
    app.exec_()



from silx.gui import qt
from Orange.canvas import __main__ as main

try:
    from orangecanvas import config
except ImportError:
    from Orange.canvas import config

    old_config = True
else:
    old_config = False
from .splash import splash_screen, getIcon
import os, sys
from Orange.misc import environ
import pkg_resources


def version():
    return pkg_resources.get_distribution("est").version


if old_config:

    class EstConfig(config.Config):
        ApplicationName = "est"
        ApplicationVersion = version()

        @staticmethod
        def splash_screen():
            return splash_screen()

        @staticmethod
        def core_packages():
            return super(EstConfig, EstConfig).core_packages() + ["est-add-on"]

        @staticmethod
        def application_icon(self):
            return getIcon()


else:

    class EstConfig(config.Config):
        ApplicationName = "est"
        ApplicationVersion = version()

        def splash_screen(self):
            return splash_screen()

        def core_packages(self):
            return super(EstConfig, EstConfig).core_packages() + ["est-add-on"]

        def application_icon(self):
            return getIcon()


class EstSplashScreen(qt.QSplashScreen):
    """SplashScreen to overwrite the one of Orange"""

    def __init__(
        self,
        parent=None,
        pixmap=None,
        textRect=None,
        textFormat=qt.Qt.PlainText,
        **kwargs
    ):
        qt.QSplashScreen.__init__(self, parent, pixmap=pixmap, **kwargs)

    def showMessage(self, message, alignment=qt.Qt.AlignLeft, color=qt.Qt.black):
        super().showMessage(
            version(), qt.Qt.AlignCenter | qt.Qt.AlignBottom, qt.QColor("#e68f17")
        )


class Launcher:
    """Proxy to orange-canvas"""

    def launch(self, argv):
        config.Config = EstConfig
        self.fix_application_dirs()
        self.replace_splash_screen()
        self.main(argv)

    def fix_application_dirs(self):
        def data_dir(versioned=True):
            """
            Return the platform dependent Orange data directory.

            This is ``data_dir_base()``/Orange/__VERSION__/ directory if versioned is
            `True` and ``data_dir_base()``/Orange/ otherwise.
            """
            base = environ.data_dir_base()
            if versioned:
                return os.path.join(base, "est", version())
            else:
                return os.path.join(base, "est")

        environ.data_dir = data_dir

        def cache_dir(*args):
            """
            Return the platform dependent Orange cache directory.
            """
            if sys.platform == "darwin":
                base = os.path.expanduser("~/Library/Caches")
            elif sys.platform == "win32":
                base = os.getenv("APPDATA", os.path.expanduser("~/AppData/Local"))
            elif os.name == "posix":
                base = os.getenv("XDG_CACHE_HOME", os.path.expanduser("~/.cache"))
            else:
                base = os.path.expanduser("~/.cache")

            base = os.path.join(base, "est", version())
            if sys.platform == "win32":
                # On Windows cache and data dir are the same.
                # Microsoft suggest using a Cache subdirectory
                return os.path.join(base, "Cache")
            else:
                return base

        environ.cache_dir = cache_dir

    def replace_splash_screen(self):
        main.SplashScreen = EstSplashScreen

    def main(self, argv):
        from Orange.canvas.__main__ import main

        main(argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "02/10/2018"

import logging

from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.widgets.widget import Input, Output
import Orange.data
from orangecontrib.est.utils import Converter
from silx.gui import qt

import est.core.process.roi
from est.core.types import XASObject
from est.gui.roiselector import ROISelector

_logger = logging.getLogger(__file__)


class RoiSelectionOW(OWWidget):
    """
    Widget used to make the selection of a region of Interest to treat in a
    Dataset.
    """

    name = "ROI definition"
    id = "orange.widgets.xas.utils.roiselection"
    description = "Select data Region Of Interest"
    icon = "icons/image-select-box.svg"

    priority = 10
    category = "esrfWidgets"
    keywords = ["dataset", "data", "selection", "ROI", "Region of Interest"]

    process_function = est.core.process.roi.ROIProcess

    want_main_area = True
    resizing_enabled = True
    compress_signal = False
    allows_cycle = False

    _roi_origin = Setting(tuple())
    _roi_size = Setting(tuple())

    class Inputs:
        xas_obj = Input("xas_obj", XASObject, default=True)
        # simple compatibility for some Orange widget and especialy the
        # 'spectroscopy add-on'
        data_table = Input("Data", Orange.data.Table)

    class Outputs:
        res_xas_obj = Output("xas_obj", XASObject)
        # by default we want to avoid sending 'Orange.data.Table' to avoid
        # loosing the XASObject flow process and results.

    def __init__(self):
        super().__init__()

        self._widget = ROISelector(parent=self)
        layout = gui.vBox(self.mainArea, "data selection").layout()
        layout.addWidget(self._widget)

        # buttons
        types = qt.QDialogButtonBox.Ok
        self._buttons = qt.QDialogButtonBox(parent=self)
        self._buttons.setStandardButtons(types)
        layout.addWidget(self._buttons)

        self._buttons.hide()

        # connect signal / slot
        self._buttons.accepted.connect(self.validate)

        # expose API
        self.setROI = self._widget.setROI
        self.getROI = self._widget.getROI

    @Inputs.data_table
    def processFrmDataTable(self, data_table):
        if data_table is None:
            return
        self.process(Converter.toXASObject(data_table=data_table))

    @Inputs.xas_obj
    def process(self, xas_obj):
        if xas_obj is None:
            return
        self._widget.setXasObject(xas_obj=xas_obj)
        self._buttons.show()
        self.show()

    def validate(self):
        """
        callback when the ROI has been validated
        """
        if self._widget.getXasObject() is None:
            return

        try:
            roi_process = est.core.process.roi.ROIProcess()
            xas_roi = est.core.process.roi._ROI.from_silx_def(self.getROI()).to_dict()
            roi_prop = {"roi": xas_roi}
            roi_process.set_properties(roi_prop)
            # we want to keep the raw data in roi object, to be able to resize
            # the roi (especially increase it size)
            xas_obj = roi_process.process(xas_obj=self._widget.getXasObject())
            self.Outputs.res_xas_obj.send(xas_obj)
        except Exception as e:
            _logger.error(e)
        else:
            OWWidget.accept(self)

    def updateProperties(self):
        # as dim are named tuple we have to cast them to dict
        self._roi_origin = tuple(self._widget.getROI().getOrigin())
        self._roi_size = tuple(self._widget.getROI().getSize())



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "25/01/2021"


from est.core.types import Dim
from est.units import ur


class SpecInputInformation:
    """Input information relative to spec"""

    def __init__(
        self,
        spec_file,
        energy_abs_col_name,
        input_abs_col_name,
        input_monitor_col_name,
        scan_title,
    ):
        self.__spec_file = None
        self.__scan_title = None
        self.__energy_abs_col_name = None
        self.__input_abs_col_name = None
        self.__input_monitor_col_name = None

        self.spec_file = spec_file
        self.scan_title = scan_title
        self.energy_abs_col_name = energy_abs_col_name
        self.input_abs_col_name = input_abs_col_name
        self.input_monitor_col_name = input_monitor_col_name

    @property
    def spec_file(self):
        return self.__spec_file

    @spec_file.setter
    def spec_file(self, spec_file):
        self.__spec_file = spec_file

    @property
    def scan_title(self):
        return self.__scan_title

    @scan_title.setter
    def scan_title(self, title):
        self.__scan_title = title

    @property
    def energy_abs_col_name(self):
        return self.__energy_abs_col_name

    @energy_abs_col_name.setter
    def energy_abs_col_name(self, col_name):
        self.__energy_abs_col_name = col_name

    @property
    def input_abs_col_name(self):
        return self.__input_abs_col_name

    @input_abs_col_name.setter
    def input_abs_col_name(self, col_name):
        self.__input_abs_col_name = col_name

    @property
    def input_monitor_col_name(self):
        return self.__input_monitor_col_name

    @input_monitor_col_name.setter
    def input_monitor_col_name(self, col_name):
        self.__input_monitor_col_name = col_name


class _HDF5InputInformation:
    def __init__(self):
        pass


class InputInformation:
    """
    Utils class to store input information
    """

    def __init__(
        self,
        spectra_url,
        channel_url,
        config_url=None,
        dimensions=(Dim.DIM_2, Dim.DIM_1, Dim.DIM_0),
        energy_unit=ur.eV,
        spec_input=None,
        columns_names=None,
    ):
        # main information
        self.__spectra_url = None
        self.__channel_url = None
        self.__dimensions = None
        self.__config_url = None
        self.__energy_unit = None
        self.__spec_info = None
        self.__columns_names = None

        # "fancy information"
        self.__I0_url = None
        self.__I1_url = None
        self.__I2_url = None
        self.__mu_ref_url = None

        self.spectra_url = spectra_url
        self.channel_url = channel_url
        self.config_url = config_url
        self.dimensions = dimensions
        self.energy_unit = energy_unit
        self.columns_names = columns_names

    def is_spec_input(self) -> bool:
        """return True if the input information (spectra...) comes from a
        spec file
        """
        return self.__spec_info is not None

    def has_url_information(self) -> bool:
        """
        Return True if this contains at least on of the following url:
               input_spectra_url or input_channel_url or input_configuration_url

        """
        return self.spectra_url or self.channel_url or self.configuration_url

    def is_valid(self):
        """Return True if at least a spec file is provided or some url
        information"""
        return self.spec_file is not None or self.has_url_information

    @property
    def spectra_url(self):
        return self.__spectra_url

    @spectra_url.setter
    def spectra_url(self, url):
        # TODO: maybe we want to add a check to create a DataUrl if this is a string...
        self.__spectra_url = url

    @property
    def channel_url(self):
        return self.__channel_url

    @channel_url.setter
    def channel_url(self, url):
        self.__channel_url = url

    @property
    def config_url(self):
        return self.__config_url

    @config_url.setter
    def config_url(self, url):
        self.__config_url = url

    @property
    def dimensions(self):
        return self.__dimensions

    @dimensions.setter
    def dimensions(self, dims):
        self.__dimensions = dims

    @property
    def energy_unit(self):
        return self.__energy_unit

    @energy_unit.setter
    def energy_unit(self, unit):
        self.__energy_unit = unit

    @property
    def columns_names(self):
        return self.__columns_names

    @columns_names.setter
    def columns_names(self, columns_names):
        self.__columns_names = columns_names

    @property
    def I0_url(self):
        return self.__I0_url

    @I0_url.setter
    def I0_url(self, url):
        self.__I0_url = url

    @property
    def I1_url(self):
        return self.__I1_url

    @I1_url.setter
    def I1_url(self, url):
        self.__I1_url = url

    @property
    def I2_url(self):
        return self.__I2_url

    @I2_url.setter
    def I2_url(self, url):
        self.__I2_url = url

    @property
    def mu_ref_url(self):
        return self.__mu_ref_url

    @mu_ref_url.setter
    def mu_ref_url(self, url):
        self.__mu_ref_url = url

    @property
    def spec_info(self):
        return self.__spec_info

    @property
    def scan_title(self):
        if self.is_spec_input():
            return self.spec_info.scan_title
        else:
            return None

    @property
    def scan_title(self):
        if self.is_spec_input():
            return self.spec_info.scan_title
        else:
            return None

    @property
    def spec_file(self):
        if self.is_spec_input():
            return self.spec_info.spec_file
        else:
            return None

    @property
    def input_energy_col_name(self):
        if self.is_spec_input():
            return self.spec_info.input_energy_col_name
        else:
            return None

    @property
    def input_abs_col_name(self):
        if self.is_spec_input():
            return self.spec_info.input_abs_col_name
        else:
            return None

    @property
    def input_monitor_col_name(self):
        if self.is_spec_input():
            return self.spec_info.input_monitor_col_name
        else:
            return None

    def __str__(self):
        return "\n".join(
            (
                "is spec: {}".format(self.is_spec_input()),
                "spectra url is: {}".format(self.spectra_url),
                "channel url is: {}".format(self.channel_url),
                "config url is: {}".format(self.config_url),
                "dimensions are: {}".format(self.dimensions),
                "energy unit is: {}".format(self.energy_unit),
                "columns names: {}".format(self.columns_names),
            )
        )



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "27/01/2021"

from est.core.types import XASObject
from .process import Process
from est.core.process.process import _input_desc
from est.core.process.process import _output_desc
import logging
from typing import Union

_logger = logging.getLogger(__name__)


def xas_energy_roi(xas_obj):
    """
    apply roi on the XASObject.spectra

    :param xas_obj: object containing the configuration and spectra to process
    :type: Union[:class:`.XASObject`, dict]
    :return: spectra dict
    :rtype: :class:`.XASObject`
    """
    xas_roi_process = EnergyROIProcess()
    return xas_roi_process.process(xas_obj=xas_obj)


class ROI1D:
    def __init__(self, from_, to_):
        self.from_ = from_
        self.to_ = to_


class EnergyROIProcess(Process):

    inputs = [
        _input_desc(name="xas_obj", type=XASObject, handler="process", doc=""),
    ]

    outputs = [
        _output_desc(name="xas_obj", type=XASObject, doc=""),
    ]

    def __init__(self):
        Process.__init__(self, name="energy roi")
        self._roi = ROI1D(0, 9999999999)

    def set_properties(self, properties: dict):
        if "energy_roi" in properties:
            self._settings = properties["energy_roi"]
            self._read_roi(self._settings)

    def update_properties(self, properties):
        super().update_properties(properties=properties)
        self._read_roi(properties=self._settings)

    def _read_roi(self, properties):
        if "minE" in properties:
            self._roi.from_ = properties["minE"]
        if "maxE" in properties:
            self._roi.to_ = properties["maxE"]

    def setRoi(self, from_, to_):
        self._roi = ROI1D(from_=from_, to_=to_)

    def _apply_roi(self, xas_obj, roi):
        mask = (xas_obj.spectra.energy <= roi.to_) & (
            xas_obj.spectra.energy >= roi.from_
        )
        xas_obj.spectra.energy = xas_obj.spectra.energy[mask]
        for spectrum in xas_obj.spectra.data.flat:
            spectrum.energy = spectrum.energy[mask]
            spectrum.mu = spectrum.mu[mask]
            for attached_key in ("I0", "I1", "I2", "mu_ref"):
                if hasattr(spectrum, attached_key):
                    values = getattr(spectrum, attached_key)[mask]
                    setattr(spectrum, attached_key, values)
        #     print(spectrum.__dict__)
        # print(xas_obj.__dict__)
        return xas_obj

    def process(self, xas_obj: Union[dict, XASObject]):
        """

        :param xas_obj: object containing the configuration and spectra to process
        :type: Union[:class:`.XASObject`, dict]
        :return: spectra dict
        :rtype: :class:`.XASObject`
        """
        assert xas_obj is not None
        _xas_obj = self.getXasObject(xas_obj=xas_obj)

        # existing roi is priority. This is the case if called from pushworkflow
        # for example.
        if self._roi is not None:
            xas_obj = self._apply_roi(xas_obj=_xas_obj, roi=self._roi)

        self.register_process(_xas_obj, data_keys=("Mu", "energy"))
        return xas_obj

    def definition(self):
        return "apply a ROI on energy range"

    def program_version(self):
        import est.version

        return est.version.version

    def program_name(self):
        return "energy-roi"

    __call__ = process



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import argparse
import sys
from est.gui.xas_object_definition import XASObjectWindow
from .utils import get_unit
from .utils import get_url
from .utils import convert_spectra_dims
from silx.gui import qt
from est.core.types import Dim
import signal


def _plot(
    spec_input,
    input_spectra_url,
    input_spectra_dims,
    input_channel_url,
    input_energy_unit,
):
    has_url_information = input_spectra_url or input_channel_url
    (
        input_spec_file,
        input_energy_col_name,
        input_abs_col_name,
        input_monitor_col_name,
        scan_title,
    ) = spec_input
    if input_spec_file is not None and has_url_information:
        raise ValueError("You cannot provide an input file and input urls")

    app = qt.QApplication([])
    widget = XASObjectWindow(parent=None)
    if input_spec_file is not None:
        # warning: the file should set prior to col names... because
        # they need to be defined
        if input_spec_file.lower().endswith(".xmu"):
            widget.setXmuFile(input_spec_file)
            widget.setCurrentType("*.xmu")
        elif input_spec_file.lower().endswith(".csv"):
            widget.setCsvFile("*.csv")
        else:
            widget.setDatFile(input_spec_file)
            widget.setCurrentType("*.dat")

        if input_energy_col_name is not None:
            widget.setEnergyColName(input_energy_col_name)
        if input_abs_col_name is not None:
            widget.setAbsColName(input_abs_col_name)
        if scan_title is not None:
            widget.setScanTitle(scan_title)
        if input_monitor_col_name is not None:
            widget.setMonitorColName(input_monitor_col_name)
    else:
        widget.setSpectraUrl(input_spectra_url)
        widget.setEnergyUrl(input_channel_url)
        widget.setCurrentType("*.h5")
    widget.setEnergyUnit(input_energy_unit)
    widget.setDimensions(input_spectra_dims)
    widget.loadXasObject()
    widget.show()

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler
    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catch
    timer.timeout.connect(lambda: None)

    app.exec_()


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    # single file input option
    parser.add_argument(
        "-i",
        "--input",
        dest="input_",
        default=None,
        help="Input of the workflow. Should be a path to a file",
    )
    # input url option
    parser.add_argument(
        "--input-spectra",
        "--spectra",
        dest="input_spectra",
        default=None,
        help="Input spectra url",
    )
    parser.add_argument(
        "--input-spectra-dims",
        "--spectra-dims",
        dest="input_spectra_dims",
        default=(Dim.DIM_2.value, Dim.DIM_1.value, Dim.DIM_0.value),
        help="spectra dimension. Should be a tuple of three values like: (Z,Y,X)",
    )
    parser.add_argument(
        "--input-channel",
        "--channel",
        dest="input_channel",
        default=None,
        help="Input channel url (usually energy)",
    )
    parser.add_argument(
        "--input-energy-unit",
        "--energy-unit",
        dest="input_energy_unit",
        default="eV",
        help="energy unit",
    )
    parser.add_argument(
        "--input-energy-col-name",
        "--energy-col-name",
        dest="input_energy_col_name",
        default=None,
        help="Provide name of the energy column for spec file",
    )
    parser.add_argument(
        "--input-abs-col-name",
        "--abs-col-name",
        dest="input_abs_col_name",
        default=None,
        help="Provide name of the absorption column for spec file",
    )
    parser.add_argument(
        "--input-monitor-col-name",
        "--monitor-col-name",
        dest="input_monitor_col_name",
        default=None,
        help="Provide name of the monitor column for spec file",
    )
    parser.add_argument(
        "--input-scan-title",
        "--scan-title",
        dest="input_scan_title_name",
        default=None,
        help="Provide scan title name to consider",
    )
    options = parser.parse_args(argv[1:])

    _plot(
        spec_input=(
            options.input_,
            options.input_energy_col_name,
            options.input_abs_col_name,
            options.input_monitor_col_name,
            options.input_scan_title_name,
        ),
        input_spectra_url=get_url(options.input_spectra),
        input_spectra_dims=convert_spectra_dims(options.input_spectra_dims),
        input_channel_url=get_url(options.input_channel),
        input_energy_unit=get_unit(options.input_energy_unit),
    )


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Tools for noise calculation"""


__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "18/01/2021"


from silx.gui import qt
from est.gui.XasObjectViewer import SpectrumViewer
from est.gui.XasObjectViewer import _plot_noise_savgol, _plot_norm_noise_savgol
import numpy


class _QOddSpinBox(qt.QSpinBox):
    """
    SpinBox dedicated to odd numbers
    """

    oddValueChanged = qt.Signal(int)

    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.valueChanged.connect(self.onValueChanged)
        self.before_value = self.value()

    def onValueChanged(self, i):
        if not self.isValid(i):
            self.setValue(self.before_value)
        else:
            self.oddValueChanged.emit(i)
            self.before_value = i

    def isValid(self, value):
        if (value % 2) == 1:
            return True
        return False


class _SavitskyGolayNoiseOpts(qt.QWidget):
    sigChanged = qt.Signal()
    """signal emitted when parameters change"""

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QFormLayout())

        ## Inputs
        self.layout().addRow(qt.QLabel("Inputs", self))

        # add window size
        self._windowSize = _QOddSpinBox(self)
        self._windowSize.setValue(5)
        self._windowSize.setSingleStep(2)
        self.layout().addRow(qt.QLabel("     - window size", self), self._windowSize)
        self._windowSize.setToolTip(
            "window size to provide to the Savitsky-Golay algorithm"
        )
        # add polynomial order
        self._polynomialOrder = qt.QSpinBox(self)
        self._polynomialOrder.setValue(2)
        self.layout().addRow(
            qt.QLabel("     - polynomial order", self), self._polynomialOrder
        )
        self._polynomialOrder.setToolTip(
            "window size to provide to the Savitsky-Golay algorithm"
        )
        # E start
        self._eStart = qt.QDoubleSpinBox(self)
        self._eStart.setRange(0, 10000000001)
        self._eStart.setSuffix("eV")
        self._eStart.setValue(100)
        self.layout().addRow(qt.QLabel("     - E start", self), self._eStart)

        # E end
        self._eEnd = qt.QDoubleSpinBox(self)
        self._eEnd.setRange(0, 10000000001)
        self._eEnd.setSuffix("eV")
        self._eEnd.setValue(10000000000)
        self.layout().addRow(qt.QLabel("     - E end", self), self._eEnd)

        ## Outputs
        self.layout().addRow(qt.QLabel("Outputs", self))
        # raw data average noise
        self._rawData = qt.QLineEdit("", self)
        self._rawData.setReadOnly(True)
        self.layout().addRow(
            qt.QLabel("     - Raw data average noise (blue)", self), self._rawData
        )
        self._rawData.setToolTip(
            "Raw data average noise is calculated as the difference between "
            "the raw spectrum and the spectrum smoothed with Savitsky-Golay "
            "algorithm (with provided window size and polynomial order)"
        )

        # edge step
        self._edgeStep = qt.QLineEdit("", self)
        self._edgeStep.setReadOnly(True)
        self.layout().addRow(qt.QLabel("     - edge step", self), self._edgeStep)
        self._edgeStep.setToolTip(
            "Edge step is defined during the normalization (pymca) or the "
            "pre-edge step (larch)"
        )

        # normalized noise
        self._normalizedNoise = qt.QLineEdit("", self)
        self._normalizedNoise.setReadOnly(True)
        self.layout().addRow(
            qt.QLabel("     - Normalized noise (red)", self), self._normalizedNoise
        )
        self._normalizedNoise.setToolTip("Average noise divided by edge jump")

        # connect signal / slot
        self._windowSize.oddValueChanged.connect(self._changed)
        self._polynomialOrder.valueChanged.connect(self._changed)

    def getWindowSize(self):
        return self._windowSize.value()

    def setWindowSize(self, size):
        if size < 1:
            return
        self._windowSize.setValue(size)

    def getPolynomialOrder(self):
        return self._polynomialOrder.value()

    def setPolynomialOrder(self, order):
        if order < 1:
            return
        self._polynomialOrder.setValue(order)

    def getEStart(self):
        return self._eStart.value()

    def setEStart(self, e_min):
        if e_min > 0:
            self._eStart.setValue(e_min)

    def getEEnd(self):
        return self._eEnd.value()

    def setEEnd(self, e_max):
        if e_max > 0:
            self._eEnd.setValue(e_max)

    def getParameters(self):
        return {
            "window_size": self.getWindowSize(),
            "polynomial_order": self.getPolynomialOrder(),
            "e_min": self.getEStart(),
            "e_max": self.getEEnd(),
        }

    def setParameters(self, config):
        if "window_size" in config:
            self.setWindowSize(config["window_size"])
        if "polynomial_order" in config:
            self.setPolynomialOrder(config["polynomial_order"])
        if "e_min" in config:
            self.setEStart(config["e_min"])
        if "e_max" in config:
            self.setEEnd(config["e_max"])

    def _changed(self, *args, **kwargs):
        self.sigChanged.emit()

    def setSpectrum(self, spectrum):
        if spectrum.noise_savgol is not None:
            self._rawData.setText("{:.5E}".format(numpy.mean(spectrum.noise_savgol)))
        else:
            self._rawData.clear()
        if hasattr(spectrum, "edge_step"):
            self._edgeStep.setText("{:.3f}".format(spectrum.edge_step))
        else:
            self._edgeStep.setText("?")
        if spectrum.norm_noise_savgol is not None:
            self._normalizedNoise.setText("{:.5E}".format(spectrum.norm_noise_savgol))
        else:
            self._normalizedNoise.clear()


class SavitskyGolayNoise(qt.QMainWindow):
    """
    Widget to tune SavitskyGolayNoise and display the results
    """

    sigChanged = qt.Signal()
    """signal emitted when parameters change"""

    def __init__(self, parent):
        qt.QMainWindow.__init__(self, parent)
        self.setWindowFlags(qt.Qt.Widget)

        # define centre widget
        self._plot = SpectrumViewer()
        self._plot.setYAxisLogarithmic(True)
        for ope in (_plot_noise_savgol, _plot_norm_noise_savgol):
            self._plot.addCurveOperation(ope)
        self.setCentralWidget(self._plot)

        # options
        self._options = _SavitskyGolayNoiseOpts(parent=self)
        self._optionsDW = qt.QDockWidget(self)
        self._optionsDW.setWidget(self._options)
        self._optionsDW.setAllowedAreas(
            qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea
        )
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self._optionsDW)
        self._optionsDW.setFeatures(qt.QDockWidget.DockWidgetMovable)

        # connect signal / plot
        self._options.sigChanged.connect(self._changed)
        self._plot.sigSpectrumChanged.connect(self._updateSpectrumInfo)

    def setXASObj(self, xas_obj):
        self._plot.setXasObject(xas_obj=xas_obj)

    def getParameters(self):
        return self._options.getParameters()

    def setParameters(self, config):
        self._options.setParameters(config=config)

    def getWindowSize(self):
        return self._options.getWindowSize()

    def setWindowSize(self, size):
        self._options.setWindowSize(size=size)

    def getPolynomialOrder(self):
        return self._options.getPolynomialOrder()

    def setPolynomialOrder(self, order):
        self._options.setPolynomialOrder(order=order)

    def _changed(self, *args, **kwargs):
        self.sigChanged.emit()

    def _updateSpectrumInfo(self):
        current_spectrum = self._plot.getCurrentSpectrum()
        if current_spectrum is not None:
            self._options.setSpectrum(spectrum=current_spectrum)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"

import shutil
import tempfile
import h5py
import numpy
import os
import unittest
from est.core.io import XASWriter
from est.core.io import read as read_xas
from est.core.utils import spectra as spectra_utils
from est.core.types import XASObject
from silx.io.url import DataUrl
from est.core.types import Dim

try:
    import PyMca5
except ImportError:
    has_pymca = False
else:
    has_pymca = True
    from PyMca5.PyMcaDataDir import PYMCA_DATA_DIR


class TestReadWrite(unittest.TestCase):
    """Test read function for spectra and configuration"""

    def setUp(self):
        self.outputdir = tempfile.mkdtemp()
        self.outputfile = os.path.join(self.outputdir, "output_file.h5")

    def tearDown(self):
        shutil.rmtree(self.outputdir)

    @unittest.skipIf(has_pymca is False, "PyMca5 is not installed")
    def testReadSpectrum(self):
        data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")
        res = read_xas(
            spectra_url=DataUrl(
                file_path=data_file, scheme="PyMca", data_path="Column 1"
            ),
            channel_url=DataUrl(
                file_path=data_file, scheme="PyMca", data_path="Column 2"
            ),
        )
        self.assertTrue(isinstance(res, XASObject))
        self.assertEqual(res.n_spectrum, 1)
        self.assertTrue("Mu" in res.spectra.data.flat[0])
        self.assertTrue("Energy" in res.spectra.data.flat[0])


class TestNxWriting(unittest.TestCase):
    """Test that the nx process is correctly store ad the output data"""

    def setUp(self):
        self.output_dir = tempfile.mkdtemp()
        energy, spectra = spectra_utils.create_dataset(shape=(256, 20, 10))
        self.xas_obj = XASObject(spectra=spectra, energy=energy, dim1=20, dim2=10)
        self.h5_file = os.path.join(self.output_dir, "output_file.h5")

    def tearDown(self):
        shutil.rmtree(self.output_dir)

    def testWriteRead(self):
        writer = XASWriter()
        writer.output_file = self.h5_file
        writer(self.xas_obj)
        with h5py.File(self.h5_file, "r") as hdf:
            self.assertTrue("scan1" in hdf.keys())
            self.assertTrue("data" in hdf["scan1"].keys())
            self.assertTrue("absorbed_beam" in hdf["scan1"].keys())
            self.assertTrue("monochromator" in hdf["scan1"].keys())

        loaded_xas_obj = XASObject.from_file(self.h5_file, configuration_path=None)
        numpy.testing.assert_allclose(loaded_xas_obj.energy, self.xas_obj.energy)
        numpy.testing.assert_allclose(
            loaded_xas_obj.absorbed_beam(), self.xas_obj.absorbed_beam()
        )


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestReadWrite, TestNxWriting):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Tool to display Spectra maps and estimate E0"""


__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "04/12/2019"


from silx.gui import qt
from silx.utils.enum import Enum
from est.core.types import XASObject
from silx.gui.plot.items.roi import RectangleROI
from silx.gui.plot.tools.roi import RegionOfInterestManager
import functools
import numpy

from est.gui.XasObjectViewer import MapViewer


class E0ComputationMethod(Enum):
    """Methods to compute E0"""

    MEAN = "mean"
    MEDIAN = "median"


class _ComputationScale(Enum):
    ALL = "all"
    CURRENT = "current"


class E0CalculatorDialog(qt.QDialog):
    """Dialog to compute E0 mean or median in a ROI"""

    def __init__(self, parent=None, xas_obj=None):
        qt.QDialog.__init__(self, parent=parent)
        if xas_obj is None:
            raise ValueError("xas_obj should not be None")

        self._methodToUse = None
        # set when the user quit the dialog and define to use mean or median

        # add the E0Calculator
        self._mainWidget = E0Calculator(parent=self, xas_obj=xas_obj)
        self.setLayout(qt.QVBoxLayout())
        self.layout().addWidget(self._mainWidget)

        # add the buttons
        style = qt.QApplication.instance().style()
        icon = style.standardIcon(qt.QStyle.SP_DialogApplyButton)

        _buttons = qt.QDialogButtonBox(parent=self)
        self._useMedian = qt.QPushButton(icon, "use median", self)
        _buttons.addButton(self._useMedian, qt.QDialogButtonBox.ActionRole)
        self._useMean = qt.QPushButton(icon, "use mean", self)
        _buttons.addButton(self._useMean, qt.QDialogButtonBox.ActionRole)
        _buttons.addButton(qt.QDialogButtonBox.Cancel)
        self.layout().addWidget(_buttons)

        # connect signal / slot
        _buttons.rejected.connect(self.reject)
        self._useMean.released.connect(
            functools.partial(self.validateMethodToUse, E0ComputationMethod.MEAN)
        )
        self._useMedian.released.connect(
            functools.partial(self.validateMethodToUse, E0ComputationMethod.MEDIAN)
        )

    def validateMethodToUse(self, method):
        """Define the method to use and close the dialog"""
        method = E0ComputationMethod.from_value(method)
        assert method in (None, E0ComputationMethod.MEDIAN, E0ComputationMethod.MEAN)
        self._methodToUse = method
        self.accept()

    def getE0(self):
        if self._methodToUse is None:
            return None
        else:
            return self._mainWidget.getE0(method=self._methodToUse)


class E0Calculator(qt.QMainWindow):
    """Interface to compute E0 mean and median values from a ROI"""

    sigE0Changed = qt.Signal(float)
    """Signal emitted when E0 is changing"""

    def __init__(self, parent=None, xas_obj=None):
        assert isinstance(xas_obj, (XASObject, type(None)))
        self._xas_obj = None

        qt.QMainWindow.__init__(self, parent=parent)
        # map view
        self._muView = MapViewer(parent=self, keys=("Mu",))
        self._muView.menuBar().hide()
        self._muView.keySelectionDocker.hide()
        # for now we don't wan't to manage the roi depending on the dimension.
        self._muView.setPerspectiveVisible(False)
        self.setCentralWidget(self._muView)
        self.setWindowFlags(qt.Qt.Widget)

        # add a ROI
        self._roiManager = RegionOfInterestManager(self._muView.getPlot())
        self._roiManager.setColor("red")  # Set the color of ROI

        self._roi = RectangleROI()
        self._updateRoi(xas_obj=xas_obj)
        self._roi.setName("ROI")
        self._roi.setEditable(True)
        self._roiManager.addRoi(self._roi)

        # dock widget to compute E0
        self._dockWidget = qt.QDockWidget(parent=self)
        self._e0CalculationWidget = _E0CalculationWidget(parent=self._dockWidget)
        self._dockWidget.setWidget(self._e0CalculationWidget)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self._dockWidget)
        self._dockWidget.setAllowedAreas(
            qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea
        )
        self._dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # expose scale
        self.getComputationScale = self._e0CalculationWidget.getComputationScale
        self.getE0 = self._e0CalculationWidget.getE0

        # connect Signal / Slot
        self._roi.sigRegionChanged.connect(functools.partial(self._updateE0, "ROI"))
        self._muView.sigFrameChanged.connect(functools.partial(self._updateE0, "frame"))
        self._e0CalculationWidget.sigComputationScaleChanged.connect(
            functools.partial(self._updateE0, "scale")
        )

        # set up
        if xas_obj is not None:
            self.setXasObject(xas_obj=xas_obj)

    def setXasObject(self, xas_obj, update_roi=True):
        """
        set XasObject to the E0 calculator

        :param xas_obj: dataset
        :param bool update_roi: if True, fir the ROI to the XAS map
        """
        self._xas_obj = xas_obj
        self._muView.setXasObject(xas_obj=self._xas_obj)
        if update_roi:
            self._updateRoi(self._xas_obj)
        # compute E0 for the current roi
        self._updateE0(origin="init")

    def getXasObject(self):
        return self._xas_obj

    def _updateRoi(self, xas_obj):
        if xas_obj is not None:
            dim_1, dim_2 = xas_obj.dim1, xas_obj.dim2
        else:
            dim_1, dim_2 = 100, 100
        self._roi.setGeometry(origin=(0, 0), size=(dim_2, dim_1))

    def _updateE0(self, origin):
        """

        :param origin: what is the reason of updating E0:
                       * `init`: construction of the widget, we want to run the
                         the first computation
                       * `frame`: the current frame change
                       * `scale`: the scale to compute E0 changed
                       * `ROI`: the region of interest changed
        :type: str
        :return:
        """
        assert origin in ("ROI", "frame", "scale", "init")
        if origin == "frame" and self.getComputationScale() == _ComputationScale.ALL:
            return
        if self._xas_obj is None:
            return

        if self.getComputationScale() == _ComputationScale.ALL:
            data = self._xas_obj.spectra.map_to("mu")
        else:
            data = self._muView.getActiveImage(just_legend=False).getData()

        e0 = self._compute0(roi=self._roi, data=data)
        self._e0CalculationWidget.setE0(e0)

    def _compute0(self, roi, data):
        if self._xas_obj is None:
            return
        roi_origin = roi.getOrigin()
        roi_size = roi.getSize()
        ymin, ymax = int(roi_origin[1]), int(roi_origin[1] + roi_size[1])
        xmin, xmax = int(roi_origin[0]), int(roi_origin[0] + roi_size[0])
        # clip roi
        ymin = max(ymin, 0)
        ymax = min(ymax, self._xas_obj.dim1)
        xmin = max(xmin, 0)
        xmax = min(xmax, self._xas_obj.dim2)

        if data.ndim == 3:
            # TODO: take into account the dimension displayed
            roi_data = data[:, ymin:ymax, xmin:xmax]
        elif data.ndim:
            roi_data = data[ymin:ymax, xmin:xmax]
        else:
            raise ValueError("given data dim %s is not managed" % data.ndim)

        return {
            E0ComputationMethod.MEAN: numpy.mean(roi_data),
            E0ComputationMethod.MEDIAN: numpy.median(roi_data),
        }


class _E0CalculationWidget(qt.QGroupBox):
    """Contains all the options to compute E0 and display the mean and the
    median values"""

    sigComputationScaleChanged = qt.Signal(str)
    """Signal emitted when the computation scale change"""

    def __init__(self, parent):
        qt.QGroupBox.__init__(self, parent=parent)
        self.setTitle("E0")
        self.setLayout(qt.QGridLayout())
        # median
        self.layout().addWidget(qt.QLabel("median", self), 0, 0)
        self._medianLE = qt.QLineEdit("0", parent=self)
        self._medianLE.setReadOnly(True)
        self.layout().addWidget(self._medianLE, 0, 1)
        # mean
        self.layout().addWidget(qt.QLabel("mean", self), 1, 0)
        self._meanLE = qt.QLineEdit("0", parent=self)
        self._meanLE.setReadOnly(True)
        self.layout().addWidget(self._meanLE, 1, 1)
        # calculation scale option
        self._computationScale = _E0CalculationScale(parent=self)
        self.layout().addWidget(self._computationScale, 2, 0, 1, 2)
        # spacer
        self._spacer = qt.QWidget(parent=self)
        self._spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(self._spacer, 3, 0)

        # expose API
        self.getComputationScale = self._computationScale.getComputationScale

        # connect signal / slot
        self._computationScale.sigComputationScaleChanged.connect(
            self._computationScaleChanged
        )

    def getE0(self, method):
        """
        Return the current E0 value from the requested computation method

        :param method: method of compuation to get E0
        :type: Union[str,ComputationMethod]
        :return: E0
        :rtype: float
        """
        method = E0ComputationMethod.from_value(method)
        if method is E0ComputationMethod.MEAN:
            return float(self._meanLE.text())
        elif method is E0ComputationMethod.MEDIAN:
            return float(self._medianLE.text())
        else:
            raise ValueError("Given method %s is not managed" % method)

    def setE0(self, values):
        """

        :param values: dictionary with the different ComputationMethod as keys
                       and the value computed as values
        :type: dict
        :return:
        """
        assert E0ComputationMethod.MEDIAN in values
        assert E0ComputationMethod.MEAN in values
        self._meanLE.setText("%.3f" % values[E0ComputationMethod.MEAN])
        self._medianLE.setText("%.3f" % values[E0ComputationMethod.MEDIAN])

    def _computationScaleChanged(self, scale):
        self.sigComputationScaleChanged.emit(scale)


class _E0CalculationScale(qt.QGroupBox):
    """Interface to allow the user to select a the scale (from one frame or all
    frames) to compute E0"""

    sigComputationScaleChanged = qt.Signal(str)
    """Signal emitted when the computation scale change"""

    def __init__(self, parent):
        qt.QGroupBox.__init__(self, parent)
        self.__buttonGrp = qt.QButtonGroup(parent=self)
        self.setLayout(qt.QVBoxLayout())
        self.setTitle("compute E0 from:")
        self._frmCurrentFrameRB = qt.QRadioButton("from current frame", parent=self)
        self.layout().addWidget(self._frmCurrentFrameRB)
        self.__buttonGrp.addButton(self._frmCurrentFrameRB)
        self._frmAllMapsRB = qt.QRadioButton("from all frames", parent=self)
        self.layout().addWidget(self._frmAllMapsRB)
        self.__buttonGrp.addButton(self._frmAllMapsRB)

        # set up
        self.__buttonGrp.setExclusive(True)
        self._frmCurrentFrameRB.setChecked(True)

        # signal / slot connection
        self.__buttonGrp.buttonReleased.connect(self._statusChanged)

    def _statusChanged(self, *arg, **kwargs):
        self.sigComputationScaleChanged.emit(self.getComputationScale().value)

    def getComputationScale(self):
        if self._frmAllMapsRB.isChecked():
            return _ComputationScale.ALL
        elif self._frmCurrentFrameRB.isChecked():
            return _ComputationScale.CURRENT
        else:
            raise ValueError("At least all frame or current frame should be active")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"


import os
import unittest
from est.core.types import XASObject, Spectrum
from est.core.types import Dim

try:
    import PyMca5
except ImportError:
    has_pymca = False
else:
    has_pymca = True
    from PyMca5.PyMcaDataDir import PYMCA_DATA_DIR
    from est.core.process.pymca.exafs import pymca_exafs
    from est.core.process.pymca.ft import pymca_ft
    from est.core.process.pymca.k_weight import pymca_k_weight
    from est.core.process.pymca.normalization import pymca_normalization
    from est.io.utils.spec import read_spectrum


@unittest.skipIf(has_pymca is False, "PyMca5 is not installed")
class TestFTSingleSpectrum(unittest.TestCase):
    """Make sure the process have valid io"""

    def setUp(self):
        data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")
        energy, mu = read_spectrum(
            data_file,
            energy_col_name="Column 1",
            absorption_col_name="Column 2",
        )
        self.spectrum = Spectrum(energy=energy, mu=mu)
        self.xas_obj = XASObject(
            energy=energy, spectra=(self.spectrum,), dim1=1, dim2=1
        )
        # preproc xas obj
        self.xas_obj = pymca_normalization(xas_obj=self.xas_obj)
        self.xas_obj = pymca_exafs(xas_obj=self.xas_obj)
        self.xas_obj = pymca_k_weight(xas_obj=self.xas_obj)

    def testPyMcaXASAsInput(self):
        res = pymca_ft(xas_obj=self.xas_obj)
        self.assertTrue(isinstance(res, XASObject))
        self.assertTrue("FTRadius" in res.spectra.data.flat[0]["FT"])
        self.assertTrue("FTImaginary" in res.spectra.data.flat[0]["FT"])
        self.assertTrue("FTIntensity" in res.spectra.data.flat[0]["FT"])

    def testDictAsInput(self):
        res = pymca_ft(xas_obj=self.xas_obj.to_dict())
        self.assertTrue(isinstance(res, XASObject))
        self.assertTrue("FTRadius" in res.spectra.data.flat[0]["FT"])
        self.assertTrue("FTImaginary" in res.spectra.data.flat[0]["FT"])
        self.assertTrue("FTIntensity" in res.spectra.data.flat[0]["FT"])


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestFTSingleSpectrum,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"


import logging
import os

from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.widgets.widget import Output
from silx.gui import qt
from silx.io.url import DataUrl

import est.core.io
from est.core.types import XASObject
from est.gui.xas_object_definition import XASObjectWindow

_logger = logging.getLogger(__file__)


_DEBUG = False


class XASInputOW(OWWidget):
    """
    Widget used for signal extraction
    """

    name = "xas input"
    id = "orange.widgets.xas.utils.xas_input"
    description = "Read .dat file and convert it to spectra"
    icon = "icons/input.png"
    priority = 0
    category = "esrfWidgets"
    keywords = ["spectroscopy", "signal", "input", "file"]

    want_main_area = True
    resizing_enabled = True
    allows_cycle = False

    _spec_file_setting = Setting(tuple())
    _spectra_url_setting = Setting(str())
    _energy_url_setting = Setting(str())
    _configuration_url_setting = Setting(str())
    _dimensions_setting = Setting(tuple())
    _energy_unit_settings = Setting(str())
    _I0_url_setting = Setting(str())
    _I1_url_setting = Setting(str())
    _I2_url_setting = Setting(str())
    _mu_ref_url_setting = Setting(str())

    process_function = est.core.io.read_frm_file

    class Outputs:
        res_xas_obj = Output("xas_obj", XASObject)
        # by default we want to avoid sending 'Orange.data.Table' to avoid
        # loosing the XASObject flow process and results.

    def __init__(self):
        super().__init__()
        self._inputWindow = qt.QWidget(parent=self)
        self._inputWindow.setLayout(qt.QGridLayout())

        self._inputDialog = XASObjectWindow(parent=self)
        self._inputWindow.layout().addWidget(self._inputDialog, 0, 0, 1, 2)

        style = qt.QApplication.instance().style()
        icon = style.standardIcon(qt.QStyle.SP_DialogApplyButton)

        # add the apply button
        types = qt.QDialogButtonBox.Ok
        self._buttons = qt.QDialogButtonBox(parent=self)
        self._buttons.setStandardButtons(types)
        self.layout().addWidget(self._buttons)

        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self._inputWindow.layout().addWidget(spacer, 2, 0)

        layout = gui.vBox(self.mainArea, "input").layout()
        layout.addWidget(self._inputWindow)

        # deal with setting
        try:
            self._loadSettings()
        except:
            pass

        # expose api
        self.apply = self._emitNewFile

        # signal / slot connection
        self._buttons.accepted.connect(self.hide)
        self._buttons.accepted.connect(self._emitNewFile)
        self._inputDialog.getMainWindow().editingFinished.connect(self._storeSettings)
        self.setFileSelected = self._inputDialog.setDatFile

    def _emitNewFile(self, *args, **kwargs):
        try:
            xas_obj = self._inputDialog.buildXASObject()
        except ValueError as e:
            qt.QMessageBox.warning(self, "", str(e))
        else:
            if _DEBUG is True and xas_obj.n_spectrum > 100:
                from est.core.process.roi import ROIProcess, _ROI

                roiProcess = ROIProcess()
                roiProcess.setRoi(origin=(0, 0), size=(10, 10))
                xas_obj = roiProcess.process(xas_obj)
            self.Outputs.res_xas_obj.send(xas_obj)

    def _loadSettings(self):
        input_type = est.io.InputType.hdf5_spectra

        if len(self._spec_file_setting) == 0:
            input_spec_file = None
            input_energy_col_name = None
            input_abs_col_name = None
            input_monitor_col_name = None
            scan_title = None
        else:
            (
                input_spec_file,
                input_energy_col_name,
                input_abs_col_name,
                input_monitor_col_name,
                scan_title,
            ) = self._spec_file_setting
        if input_spec_file is not None:
            if input_spec_file.endswith(".xmu"):
                input_type = est.io.InputType.xmu_spectrum
                self._inputDialog.setXmuFile(input_spec_file)
            else:
                input_type = est.io.InputType.dat_spectrum
                self._inputDialog.setDatFile(input_spec_file)
                if input_energy_col_name is not None:
                    self._inputDialog.setEnergyColName(input_energy_col_name)
                if input_abs_col_name is not None:
                    self._inputDialog.setAbsColName(input_abs_col_name)
                if scan_title is not None:
                    self._inputDialog.setScanTitle(scan_title)
                if input_monitor_col_name is not None:
                    self._inputDialog.setMonitorColName(input_monitor_col_name)

        def load_url(url_path, setter):
            if url_path != "":
                if isinstance(url_path, DataUrl):
                    url = url_path
                else:
                    url = DataUrl(url_path)

                try:
                    if url and url.is_valid():
                        setter(url.path())
                except ... as e:
                    logging.info("fail to load ", url_path)

        load_url(self._spectra_url_setting, self._inputDialog.setSpectraUrl)
        load_url(self._energy_url_setting, self._inputDialog.setEnergyUrl)
        load_url(self._configuration_url_setting, self._inputDialog.setConfigurationUrl)

        advanceHDF5Info = self._inputDialog.getAdvanceHdf5Information()
        load_url(self._I0_url_setting, advanceHDF5Info.setI0Url)
        load_url(self._I1_url_setting, advanceHDF5Info.setI1Url)
        load_url(self._I2_url_setting, advanceHDF5Info.setI2Url)
        load_url(self._mu_ref_url_setting, advanceHDF5Info.setMuRefUrl)

        if len(self._dimensions_setting) == 3:
            self._inputDialog.setDimensions(self._dimensions_setting)
        else:
            assert len(self._dimensions_setting) == 0
        self._inputDialog.getMainWindow().setEnergyUnit(self._energy_unit_settings)

        # set up
        self._inputDialog.setCurrentType(input_type)

    def _storeSettings(self):
        # handle single file input
        self._spec_file_setting = (
            self._inputDialog.getMainWindow().getDatFile(),
            self._inputDialog.getMainWindow().getEnergyColName(),
            self._inputDialog.getMainWindow().getAbsColName(),
            self._inputDialog.getMainWindow().getMonitorColName(),
            self._inputDialog.getMainWindow().getScanTitle(),
        )

        # handle spectra
        spectra_url = self._inputDialog.getMainWindow().getSpectraUrl()
        if spectra_url is not None:
            spectra_url = spectra_url.path()
        self._spectra_url_setting = spectra_url
        # handle energy
        energy_url = self._inputDialog.getMainWindow().getEnergyUrl()
        if energy_url is not None:
            assert energy_url.is_valid()
            energy_url = energy_url.path()
        self._energy_url_setting = energy_url
        # handle configuration
        configuration_url = self._inputDialog.getMainWindow().getConfigurationUrl()
        if configuration_url is not None:
            configuration_url = configuration_url.path()
        self._configuration_url_setting = configuration_url
        # save settings
        self._dimensions_setting = self._inputDialog.getMainWindow().getDimensions()
        # save energy
        self._energy_unit_settings = str(
            self._inputDialog.getMainWindow().getEnergyUnit()
        )
        # handle extra information like I0...
        advanceHDF5Info = self._inputDialog.getAdvanceHdf5Information()
        i0_url = advanceHDF5Info.getI0Url()
        if i0_url is not None:
            self._I0_url_setting = i0_url.path()
        i1_url = advanceHDF5Info.getI1Url()
        if i1_url is not None:
            self._I1_url_setting = i1_url.path()
        i2_url = advanceHDF5Info.getI2Url()
        if i2_url is not None:
            self._I2_url_setting = i2_url.path()
        mu_ref_url = advanceHDF5Info.getMuRefUrl()
        if mu_ref_url is not None:
            self._mu_ref_url_setting = mu_ref_url.path()

    def sizeHint(self):
        return qt.QSize(400, 200)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import argparse
from pypushflow.representation.scheme.ows_parser import OwsParser
from pypushflow import Workflow
import est.version
import pypushflow.version
import subprocess
import os

logging.basicConfig(level=logging.DEBUG)
_logger = logging.getLogger(__name__)


def _convert(scheme, output_file, overwrite):
    """

    :param scheme:
    :param scan:
    :param timeout:
    :return:
    """
    _logger.warning("translate {} to {}".format(scheme, output_file))

    if os.path.exists(output_file):
        if overwrite is True:
            os.remove(output_file)
        else:
            raise ValueError("{} already exists.".format(output_file))

    with open(output_file, "w+") as file_:
        file_.write(_dump_info_generation())

    workflow = Workflow.ProcessableWorkflow(scheme)
    converter = Workflow.Converter(
        workflow=workflow, output_file=output_file, with_opts=True
    )
    converter.process()

    # set up workflow
    with open(output_file, mode="a") as file_:
        file_.write(_dump_executable_script_section())

    # call `black` on it if available
    try:
        # use help because there is no information regarding version
        subprocess.call(["python", "-m", "black", output_file], stdout=subprocess.PIPE)
    except Exception as e:
        _logger.error(
            "Fail to apply black on {}. Error is " "{}".format(output_file, e)
        )

    _logger.info(
        "translation finished. You can execute python {} [[--input --input-spectra --input-spectra-dims --input-channel --input-energy-unit --input-dimensions]]".format(
            output_file
        )
    )


def _dump_info_generation():
    return (
        "# This file has been generated automatically using \n"
        "# pypushflow {} and est {}\n".format(
            pypushflow.version.version, est.version.version
        )
    )


def _dump_executable_script_section():
    return """

from pypushflow.utils import IgnoreProcess

if __name__ == '__main__':
    import sys
    import argparse
    from est.app.utils import get_xas_obj
    from est.app.utils import get_url
    from est.app.utils import get_unit
    from est.app.utils import convert_spectra_dims
    from silx.io.url import DataUrl
    from est.core.types import Dim
    from est.io.utils.information import InputInformation
    from est.io.utils.information import SpecInputInformation

    parser = argparse.ArgumentParser(description=__doc__)
        # single file input option
    parser.add_argument(
        "-i",
        "--input",
        dest="input_",
        default=None,
        help="Input of the workflow. Should be a path to a file",
    )
    # input url option
    parser.add_argument(
        "--input-spectra",
        "--spectra",
        dest="input_spectra",
        default=None,
        help="Input spectra url",
    )
    parser.add_argument(
        "--input-spectra-dims",
        "--spectra-dims",
        dest="input_spectra_dims",
        default=None,
        help="Input spectra dimension. Should be a tuple of three values: "
        "(X,Y,channel). If None will take the default dimension "
        "according to the input type.",
    )
    parser.add_argument(
        "--input-energy",
        "--input-channel",
        "--channel",
        dest="input_channel",
        default=None,
        help="Input channel url (usually energy)",
    )
    parser.add_argument(
        "--input-configuration",
        "--configuration",
        dest="input_configuration",
        default=None,
        help="Input configuration url",
    )
    parser.add_argument(
        "--input-energy-unit",
        "--energy-unit",
        dest="input_energy_unit",
        default="eV",
        help="energy unit",
    )
    parser.add_argument(
        "--input-dimensions",
        "--dimensions",
        dest="input_dimensions",
        default="None",
        help="dimension of the input as (Z,Y,X) for example."
        "If None will take default unit according to the input type",
    )
    # I0, I1, I2 & mu_ref
    parser.add_argument(
        "--input-I0",
        "--I0",
        dest="input_I0",
        default="None",
        help="url to I0",
    )
    parser.add_argument(
        "--input-I1",
        "--I1",
        dest="input_I1",
        default="None",
        help="url to I1",
    )
    parser.add_argument(
        "--input-I2",
        "--I2",
        dest="input_I2",
        default="None",
        help="url to I2",
    )
    parser.add_argument(
        "--input-mu-ref",
        "--mu-ref",
        dest="input_mu_ref",
        default="None",
        help="url to mu_ref",
    )
    # spec file specific inputs
    parser.add_argument(
        "--input-energy-col-name",
        "--energy-col-name",
        dest="input_energy_col_name",
        default=None,
        help="Provide name of the energy column for spec file",
    )
    parser.add_argument(
        "--input-abs-col-name",
        "--abs-col-name",
        dest="input_abs_col_name",
        default=None,
        help="Provide name of the absorption column for spec file",
    )
    parser.add_argument(
        "--input-monitor-col-name",
        "--monitor-col-name",
        dest="input_monitor_col_name",
        default=None,
        help="Provide name of the monitor column for spec file",
    )
    parser.add_argument(
        "--input-scan-title",
        "--scan-title",
        dest="input_scan_title_name",
        default=None,
        help="Provide scan title name to consider",
    )
    # handle larch settings
    parser.add_argument(
        "--set-autobk-params",
        dest="set_autobk_params",
        default=None,
        help="set autobk settings",
    )
    parser.add_argument(
        "--set-mback-params",
        dest="set_mback_params",
        default=None,
        help="set mback settings",
    )
    parser.add_argument(
        "--set-mback-norm-params",
        dest="set_mback_norm_params",
        default=None,
        help="set mback norm settings",
    )
    parser.add_argument(
        "--set-pre-edge-params",
        dest="set_pre_edge_params",
        default=None,
        help="set pre-edge settings",
    )
    parser.add_argument(
        "--set-xftf-params",
        dest="set_xftf_params",
        default=None,
        help="set xftf settings",
    )
    # handle noise settings
    parser.add_argument(
        "--set-noise-params",
        dest="set_noise_params",
        default=None,
        help="set noise settings",
    )
    # handle output settings
    parser.add_argument(
        "--set-output-params",
        dest="set_output_params",
        default=None,
        help="set output settings",
    )
    # handle energy roi settings
    parser.add_argument(
        "--set-energyroi-params",
        dest="set_energyroi_params",
        default=None,
        help="set energy roi settings",
    )

    options = parser.parse_args(sys.argv[1:])
    
    input_information = InputInformation(
        spectra_url=get_url(options.input_spectra),
        channel_url=get_url(options.input_channel),
        dimensions=convert_spectra_dims(options.input_spectra_dims),
        config_url=get_url(options.input_configuration),
        energy_unit=get_unit(options.input_energy_unit),
        spec_input=SpecInputInformation(
            options.input_,
            options.input_energy_col_name,
            options.input_abs_col_name,
            options.input_monitor_col_name,
            options.input_scan_title_name,
        ),
    )
    input_information.I0 = get_url(options.input_I0)
    input_information.I1 = get_url(options.input_I1)
    input_information.I2 = get_url(options.input_I2)
    input_information.mu_ref = get_url(options.input_mu_ref)

    xas_obj = get_xas_obj(input_information)
    main(input_data=xas_obj, channel="xas_obj", options=options)
    """


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "workflow_file",
        help="Path to the .ows file defining the workflow to process with the"
        "provided scan",
    )
    parser.add_argument("output_file", help="Output python file")
    parser.add_argument(
        "--overwrite",
        help="Overwrite output file if exists",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="Set logging system in debug mode",
    )
    options = parser.parse_args(argv[1:])
    if not options.output_file.lower().endswith(".py"):
        options.output_file = options.output_file + ".py"
    # tune the log level
    log_level = logging.INFO
    if options.debug is True:
        log_level = logging.DEBUG

    for log_ in ("est", "pypushflow"):
        logging.getLogger(log_).setLevel(log_level)

    scheme = OwsParser.scheme_load(options.workflow_file, load_handlers=True)
    _convert(
        scheme=scheme, output_file=options.output_file, overwrite=options.overwrite
    )


if __name__ == "__main__":
    main(sys.argv)
# convert an ows file to a script calling est low level processes.
